# Generated from jac.g4 by ANTLR 4.9.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3i")
        buf.write("\u0455\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\4\64\t\64")
        buf.write("\4\65\t\65\4\66\t\66\4\67\t\67\48\t8\49\t9\4:\t:\4;\t")
        buf.write(";\4<\t<\4=\t=\4>\t>\4?\t?\4@\t@\4A\tA\4B\tB\4C\tC\4D\t")
        buf.write("D\4E\tE\4F\tF\4G\tG\4H\tH\4I\tI\4J\tJ\4K\tK\4L\tL\4M\t")
        buf.write("M\4N\tN\4O\tO\4P\tP\4Q\tQ\4R\tR\4S\tS\4T\tT\4U\tU\4V\t")
        buf.write("V\4W\tW\4X\tX\4Y\tY\4Z\tZ\4[\t[\4\\\t\\\4]\t]\4^\t^\4")
        buf.write("_\t_\4`\t`\4a\ta\4b\tb\4c\tc\4d\td\4e\te\4f\tf\4g\tg\4")
        buf.write("h\th\4i\ti\3\2\5\2\u00d4\n\2\3\2\7\2\u00d7\n\2\f\2\16")
        buf.write("\2\u00da\13\2\3\2\6\2\u00dd\n\2\r\2\16\2\u00de\3\2\3\2")
        buf.write("\3\3\3\3\3\3\3\3\5\3\u00e7\n\3\3\3\3\3\3\3\3\3\3\3\3\4")
        buf.write("\3\4\3\4\3\4\5\4\u00f2\n\4\3\5\3\5\3\5\5\5\u00f7\n\5\3")
        buf.write("\5\3\5\5\5\u00fb\n\5\3\5\3\5\3\5\5\5\u0100\n\5\3\5\3\5")
        buf.write("\5\5\u0104\n\5\3\5\3\5\3\5\5\5\u0109\n\5\3\5\3\5\5\5\u010d")
        buf.write("\n\5\3\5\3\5\3\5\5\5\u0112\n\5\3\5\3\5\5\5\u0116\n\5\5")
        buf.write("\5\u0118\n\5\3\6\3\6\3\6\3\6\3\6\3\6\3\6\5\6\u0121\n\6")
        buf.write("\3\7\3\7\3\7\5\7\u0126\n\7\3\b\3\b\3\b\3\b\5\b\u012c\n")
        buf.write("\b\3\b\3\b\3\b\3\b\3\b\3\b\3\b\5\b\u0135\n\b\3\t\3\t\3")
        buf.write("\t\5\t\u013a\n\t\3\t\3\t\3\n\3\n\7\n\u0140\n\n\f\n\16")
        buf.write("\n\u0143\13\n\3\n\5\n\u0146\n\n\3\n\3\n\7\n\u014a\n\n")
        buf.write("\f\n\16\n\u014d\13\n\3\n\5\n\u0150\n\n\3\n\3\n\3\13\3")
        buf.write("\13\3\13\3\13\3\13\3\13\5\13\u015a\n\13\3\13\3\13\3\13")
        buf.write("\5\13\u015f\n\13\3\13\3\13\5\13\u0163\n\13\3\13\3\13\5")
        buf.write("\13\u0167\n\13\3\f\3\f\3\f\3\r\3\r\3\r\3\r\3\16\3\16\3")
        buf.write("\16\3\16\3\17\3\17\3\17\3\17\3\20\3\20\7\20\u017a\n\20")
        buf.write("\f\20\16\20\u017d\13\20\3\20\3\20\3\20\3\20\5\20\u0183")
        buf.write("\n\20\3\21\3\21\5\21\u0187\n\21\3\22\3\22\5\22\u018b\n")
        buf.write("\22\3\23\3\23\3\23\3\23\3\23\3\23\3\23\3\23\3\23\3\23")
        buf.write("\3\23\3\23\5\23\u0199\n\23\3\24\3\24\3\24\3\24\3\24\3")
        buf.write("\24\3\24\3\24\3\24\3\24\5\24\u01a5\n\24\3\25\3\25\3\25")
        buf.write("\3\25\3\25\3\26\3\26\5\26\u01ae\n\26\3\26\5\26\u01b1\n")
        buf.write("\26\3\26\3\26\3\26\7\26\u01b6\n\26\f\26\16\26\u01b9\13")
        buf.write("\26\3\26\3\26\3\27\3\27\3\27\3\27\5\27\u01c1\n\27\3\30")
        buf.write("\3\30\3\30\3\30\3\30\5\30\u01c8\n\30\3\30\3\30\3\30\3")
        buf.write("\30\3\30\5\30\u01cf\n\30\7\30\u01d1\n\30\f\30\16\30\u01d4")
        buf.write("\13\30\3\30\3\30\3\30\3\30\3\30\5\30\u01db\n\30\3\30\5")
        buf.write("\30\u01de\n\30\3\31\3\31\5\31\u01e2\n\31\3\31\3\31\3\32")
        buf.write("\3\32\5\32\u01e8\n\32\3\32\3\32\3\32\5\32\u01ed\n\32\3")
        buf.write("\33\3\33\3\33\3\33\3\34\3\34\3\34\7\34\u01f6\n\34\f\34")
        buf.write("\16\34\u01f9\13\34\3\35\3\35\3\35\7\35\u01fe\n\35\f\35")
        buf.write("\16\35\u0201\13\35\3\36\3\36\7\36\u0205\n\36\f\36\16\36")
        buf.write("\u0208\13\36\3\36\3\36\3\36\5\36\u020d\n\36\3\37\3\37")
        buf.write("\3\37\3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 \3 ")
        buf.write("\3 \3 \5 \u0224\n \3!\3!\3!\3!\7!\u022a\n!\f!\16!\u022d")
        buf.write("\13!\3!\5!\u0230\n!\3\"\3\"\3\"\5\"\u0235\n\"\3#\3#\3")
        buf.write("#\3#\5#\u023b\n#\3#\3#\3#\3#\5#\u0241\n#\3#\5#\u0244\n")
        buf.write("#\3$\3$\3$\3$\3%\3%\3%\3&\3&\3&\3&\3&\3&\3&\3&\3&\3&\3")
        buf.write("&\3&\3&\3&\5&\u025b\n&\3\'\3\'\3\'\3\'\3(\3(\3)\3)\3)")
        buf.write("\3*\3*\3*\3*\3+\3+\3+\3+\3,\3,\3,\3,\5,\u0272\n,\3-\3")
        buf.write("-\3-\3-\3.\3.\3.\3.\5.\u027c\n.\3/\3/\3/\3/\5/\u0282\n")
        buf.write("/\3\60\3\60\3\60\3\61\3\61\3\61\3\62\3\62\3\62\3\63\3")
        buf.write("\63\5\63\u028f\n\63\3\63\3\63\3\63\5\63\u0294\n\63\3\64")
        buf.write("\3\64\3\64\7\64\u0299\n\64\f\64\16\64\u029c\13\64\3\65")
        buf.write("\3\65\3\65\3\65\3\65\3\65\7\65\u02a4\n\65\f\65\16\65\u02a7")
        buf.write("\13\65\5\65\u02a9\n\65\3\66\3\66\3\66\3\66\3\66\3\66\3")
        buf.write("\66\3\66\5\66\u02b3\n\66\3\67\3\67\3\67\38\38\38\78\u02bb")
        buf.write("\n8\f8\168\u02be\138\39\39\39\79\u02c3\n9\f9\169\u02c6")
        buf.write("\139\3:\3:\3:\5:\u02cb\n:\3;\3;\3;\7;\u02d0\n;\f;\16;")
        buf.write("\u02d3\13;\3<\3<\3<\3<\3<\3<\3<\3<\3<\3<\3<\3<\3<\3<\3")
        buf.write("<\3<\3<\5<\u02e6\n<\3<\3<\3<\3<\5<\u02ec\n<\3<\3<\6<\u02f0")
        buf.write("\n<\r<\16<\u02f1\7<\u02f4\n<\f<\16<\u02f7\13<\3=\3=\3")
        buf.write("=\3=\3=\3=\3=\5=\u0300\n=\3=\3=\3=\3=\5=\u0306\n=\5=\u0308")
        buf.write("\n=\3>\3>\3>\3?\3?\3?\3@\3@\3@\3@\3@\3@\5@\u0316\n@\3")
        buf.write("A\3A\3B\3B\3C\3C\3C\3C\3C\3C\3C\3C\5C\u0324\nC\3C\3C\3")
        buf.write("C\3C\3C\5C\u032b\nC\5C\u032d\nC\3D\3D\3D\3D\5D\u0333\n")
        buf.write("D\3D\3D\3D\3D\3D\5D\u033a\nD\5D\u033c\nD\3E\3E\3E\5E\u0341")
        buf.write("\nE\3E\3E\3E\3E\3E\5E\u0348\nE\3F\3F\5F\u034c\nF\3F\3")
        buf.write("F\3F\5F\u0351\nF\5F\u0353\nF\5F\u0355\nF\3G\3G\3G\3G\3")
        buf.write("H\3H\3H\3H\3I\3I\3I\3I\3J\3J\3J\5J\u0366\nJ\3K\3K\3K\3")
        buf.write("K\3K\3K\5K\u036e\nK\3K\5K\u0371\nK\3K\5K\u0374\nK\3L\3")
        buf.write("L\3L\3L\3L\3L\5L\u037c\nL\3L\5L\u037f\nL\3L\5L\u0382\n")
        buf.write("L\3M\3M\3M\3M\3M\3M\5M\u038a\nM\3M\5M\u038d\nM\3M\5M\u0390")
        buf.write("\nM\3N\3N\5N\u0394\nN\3N\3N\3O\3O\3O\3O\3O\3O\3O\3O\3")
        buf.write("O\3O\5O\u03a2\nO\3P\3P\3P\3P\7P\u03a8\nP\fP\16P\u03ab")
        buf.write("\13P\5P\u03ad\nP\3P\3P\3Q\3Q\3Q\3Q\3R\3R\5R\u03b7\nR\3")
        buf.write("R\3R\3S\3S\3S\5S\u03be\nS\3T\5T\u03c1\nT\3T\3T\5T\u03c5")
        buf.write("\nT\3U\3U\3U\3V\3V\5V\u03cc\nV\3W\3W\3W\3W\7W\u03d2\n")
        buf.write("W\fW\16W\u03d5\13W\5W\u03d7\nW\3W\3W\3X\3X\3X\3X\7X\u03df")
        buf.write("\nX\fX\16X\u03e2\13X\5X\u03e4\nX\3X\3X\3Y\3Y\3Y\3Y\3Z")
        buf.write("\3Z\3Z\3Z\3[\3[\3\\\5\\\u03f3\n\\\3\\\3\\\5\\\u03f7\n")
        buf.write("\\\3\\\3\\\3\\\3\\\3]\3]\5]\u03ff\n]\7]\u0401\n]\f]\16")
        buf.write("]\u0404\13]\3^\3^\3^\3^\3^\3^\3^\3^\5^\u040e\n^\3_\3_")
        buf.write("\3_\3`\3`\5`\u0415\n`\3`\6`\u0418\n`\r`\16`\u0419\3a\3")
        buf.write("a\3a\5a\u041f\na\3a\5a\u0422\na\6a\u0424\na\ra\16a\u0425")
        buf.write("\3b\3b\5b\u042a\nb\3b\3b\5b\u042e\nb\3c\3c\3c\5c\u0433")
        buf.write("\nc\6c\u0435\nc\rc\16c\u0436\3d\3d\3e\3e\5e\u043d\ne\3")
        buf.write("f\3f\5f\u0441\nf\3g\3g\3g\3g\5g\u0447\ng\3h\3h\5h\u044b")
        buf.write("\nh\5h\u044d\nh\3h\3h\3h\3h\3i\3i\3i\2\3vj\2\4\6\b\n\f")
        buf.write("\16\20\22\24\26\30\32\34\36 \"$&(*,.\60\62\64\668:<>@")
        buf.write("BDFHJLNPRTVXZ\\^`bdfhjlnprtvxz|~\u0080\u0082\u0084\u0086")
        buf.write("\u0088\u008a\u008c\u008e\u0090\u0092\u0094\u0096\u0098")
        buf.write("\u009a\u009c\u009e\u00a0\u00a2\u00a4\u00a6\u00a8\u00aa")
        buf.write("\u00ac\u00ae\u00b0\u00b2\u00b4\u00b6\u00b8\u00ba\u00bc")
        buf.write("\u00be\u00c0\u00c2\u00c4\u00c6\u00c8\u00ca\u00cc\u00ce")
        buf.write("\u00d0\2\17\4\2\33\34\"\"\4\2@ACC\3\2\62\65\3\2\678\3")
        buf.write("\2UV\3\2WY\3\2\37!\4\2\36\36dd\5\2\13\21\26\26,,\4\2\22")
        buf.write("\22\24\24\5\2\22\22\26\26,,\4\2\6\6\n\n\b\2\22\22\26\26")
        buf.write(",,_`bbdd\2\u04a6\2\u00d3\3\2\2\2\4\u00e2\3\2\2\2\6\u00ed")
        buf.write("\3\2\2\2\b\u0117\3\2\2\2\n\u0120\3\2\2\2\f\u0125\3\2\2")
        buf.write("\2\16\u0134\3\2\2\2\20\u0136\3\2\2\2\22\u013d\3\2\2\2")
        buf.write("\24\u0153\3\2\2\2\26\u0168\3\2\2\2\30\u016b\3\2\2\2\32")
        buf.write("\u016f\3\2\2\2\34\u0173\3\2\2\2\36\u0182\3\2\2\2 \u0186")
        buf.write("\3\2\2\2\"\u018a\3\2\2\2$\u0198\3\2\2\2&\u01a4\3\2\2\2")
        buf.write("(\u01a6\3\2\2\2*\u01ab\3\2\2\2,\u01c0\3\2\2\2.\u01dd\3")
        buf.write("\2\2\2\60\u01df\3\2\2\2\62\u01e5\3\2\2\2\64\u01ee\3\2")
        buf.write("\2\2\66\u01f2\3\2\2\28\u01fa\3\2\2\2:\u020c\3\2\2\2<\u020e")
        buf.write("\3\2\2\2>\u0223\3\2\2\2@\u0225\3\2\2\2B\u0231\3\2\2\2")
        buf.write("D\u0243\3\2\2\2F\u0245\3\2\2\2H\u0249\3\2\2\2J\u025a\3")
        buf.write("\2\2\2L\u025c\3\2\2\2N\u0260\3\2\2\2P\u0262\3\2\2\2R\u0265")
        buf.write("\3\2\2\2T\u0269\3\2\2\2V\u0271\3\2\2\2X\u0273\3\2\2\2")
        buf.write("Z\u0277\3\2\2\2\\\u027d\3\2\2\2^\u0283\3\2\2\2`\u0286")
        buf.write("\3\2\2\2b\u0289\3\2\2\2d\u028c\3\2\2\2f\u0295\3\2\2\2")
        buf.write("h\u02a8\3\2\2\2j\u02b2\3\2\2\2l\u02b4\3\2\2\2n\u02b7\3")
        buf.write("\2\2\2p\u02bf\3\2\2\2r\u02ca\3\2\2\2t\u02cc\3\2\2\2v\u02eb")
        buf.write("\3\2\2\2x\u0307\3\2\2\2z\u0309\3\2\2\2|\u030c\3\2\2\2")
        buf.write("~\u0315\3\2\2\2\u0080\u0317\3\2\2\2\u0082\u0319\3\2\2")
        buf.write("\2\u0084\u032c\3\2\2\2\u0086\u033b\3\2\2\2\u0088\u0340")
        buf.write("\3\2\2\2\u008a\u0354\3\2\2\2\u008c\u0356\3\2\2\2\u008e")
        buf.write("\u035a\3\2\2\2\u0090\u035e\3\2\2\2\u0092\u0365\3\2\2\2")
        buf.write("\u0094\u0373\3\2\2\2\u0096\u0381\3\2\2\2\u0098\u038f\3")
        buf.write("\2\2\2\u009a\u0391\3\2\2\2\u009c\u03a1\3\2\2\2\u009e\u03a3")
        buf.write("\3\2\2\2\u00a0\u03b0\3\2\2\2\u00a2\u03b4\3\2\2\2\u00a4")
        buf.write("\u03bd\3\2\2\2\u00a6\u03c0\3\2\2\2\u00a8\u03c6\3\2\2\2")
        buf.write("\u00aa\u03c9\3\2\2\2\u00ac\u03cd\3\2\2\2\u00ae\u03da\3")
        buf.write("\2\2\2\u00b0\u03e7\3\2\2\2\u00b2\u03eb\3\2\2\2\u00b4\u03ef")
        buf.write("\3\2\2\2\u00b6\u03f2\3\2\2\2\u00b8\u0402\3\2\2\2\u00ba")
        buf.write("\u040d\3\2\2\2\u00bc\u040f\3\2\2\2\u00be\u0417\3\2\2\2")
        buf.write("\u00c0\u0423\3\2\2\2\u00c2\u0429\3\2\2\2\u00c4\u0434\3")
        buf.write("\2\2\2\u00c6\u0438\3\2\2\2\u00c8\u043a\3\2\2\2\u00ca\u043e")
        buf.write("\3\2\2\2\u00cc\u0442\3\2\2\2\u00ce\u044c\3\2\2\2\u00d0")
        buf.write("\u0452\3\2\2\2\u00d2\u00d4\5\6\4\2\u00d3\u00d2\3\2\2\2")
        buf.write("\u00d3\u00d4\3\2\2\2\u00d4\u00d8\3\2\2\2\u00d5\u00d7\5")
        buf.write("\4\3\2\u00d6\u00d5\3\2\2\2\u00d7\u00da\3\2\2\2\u00d8\u00d6")
        buf.write("\3\2\2\2\u00d8\u00d9\3\2\2\2\u00d9\u00dc\3\2\2\2\u00da")
        buf.write("\u00d8\3\2\2\2\u00db\u00dd\5\f\7\2\u00dc\u00db\3\2\2\2")
        buf.write("\u00dd\u00de\3\2\2\2\u00de\u00dc\3\2\2\2\u00de\u00df\3")
        buf.write("\2\2\2\u00df\u00e0\3\2\2\2\u00e0\u00e1\7\2\2\3\u00e1\3")
        buf.write("\3\2\2\2\u00e2\u00e3\7#\2\2\u00e3\u00e6\7*\2\2\u00e4\u00e7")
        buf.write("\5\b\5\2\u00e5\u00e7\7W\2\2\u00e6\u00e4\3\2\2\2\u00e6")
        buf.write("\u00e5\3\2\2\2\u00e7\u00e8\3\2\2\2\u00e8\u00e9\7+\2\2")
        buf.write("\u00e9\u00ea\7\32\2\2\u00ea\u00eb\7`\2\2\u00eb\u00ec\7")
        buf.write("\60\2\2\u00ec\5\3\2\2\2\u00ed\u00ee\7\3\2\2\u00ee\u00ef")
        buf.write("\7$\2\2\u00ef\u00f1\7`\2\2\u00f0\u00f2\7\60\2\2\u00f1")
        buf.write("\u00f0\3\2\2\2\u00f1\u00f2\3\2\2\2\u00f2\7\3\2\2\2\u00f3")
        buf.write("\u00f6\7-\2\2\u00f4\u00f7\7W\2\2\u00f5\u00f7\5\n\6\2\u00f6")
        buf.write("\u00f4\3\2\2\2\u00f6\u00f5\3\2\2\2\u00f7\u00fa\3\2\2\2")
        buf.write("\u00f8\u00f9\7S\2\2\u00f9\u00fb\5\b\5\2\u00fa\u00f8\3")
        buf.write("\2\2\2\u00fa\u00fb\3\2\2\2\u00fb\u0118\3\2\2\2\u00fc\u00ff")
        buf.write("\7\26\2\2\u00fd\u0100\7W\2\2\u00fe\u0100\5\n\6\2\u00ff")
        buf.write("\u00fd\3\2\2\2\u00ff\u00fe\3\2\2\2\u0100\u0103\3\2\2\2")
        buf.write("\u0101\u0102\7S\2\2\u0102\u0104\5\b\5\2\u0103\u0101\3")
        buf.write("\2\2\2\u0103\u0104\3\2\2\2\u0104\u0118\3\2\2\2\u0105\u0108")
        buf.write("\7,\2\2\u0106\u0109\7W\2\2\u0107\u0109\5\n\6\2\u0108\u0106")
        buf.write("\3\2\2\2\u0108\u0107\3\2\2\2\u0109\u010c\3\2\2\2\u010a")
        buf.write("\u010b\7S\2\2\u010b\u010d\5\b\5\2\u010c\u010a\3\2\2\2")
        buf.write("\u010c\u010d\3\2\2\2\u010d\u0118\3\2\2\2\u010e\u0111\7")
        buf.write("\22\2\2\u010f\u0112\7W\2\2\u0110\u0112\5\n\6\2\u0111\u010f")
        buf.write("\3\2\2\2\u0111\u0110\3\2\2\2\u0112\u0115\3\2\2\2\u0113")
        buf.write("\u0114\7S\2\2\u0114\u0116\5\b\5\2\u0115\u0113\3\2\2\2")
        buf.write("\u0115\u0116\3\2\2\2\u0116\u0118\3\2\2\2\u0117\u00f3\3")
        buf.write("\2\2\2\u0117\u00fc\3\2\2\2\u0117\u0105\3\2\2\2\u0117\u010e")
        buf.write("\3\2\2\2\u0118\t\3\2\2\2\u0119\u011a\7%\2\2\u011a\u0121")
        buf.write("\7d\2\2\u011b\u011c\7%\2\2\u011c\u011d\7*\2\2\u011d\u011e")
        buf.write("\5\66\34\2\u011e\u011f\7+\2\2\u011f\u0121\3\2\2\2\u0120")
        buf.write("\u0119\3\2\2\2\u0120\u011b\3\2\2\2\u0121\13\3\2\2\2\u0122")
        buf.write("\u0126\5\16\b\2\u0123\u0126\5\20\t\2\u0124\u0126\5\24")
        buf.write("\13\2\u0125\u0122\3\2\2\2\u0125\u0123\3\2\2\2\u0125\u0124")
        buf.write("\3\2\2\2\u0126\r\3\2\2\2\u0127\u0128\7\26\2\2\u0128\u012b")
        buf.write("\7d\2\2\u0129\u012a\7$\2\2\u012a\u012c\7b\2\2\u012b\u0129")
        buf.write("\3\2\2\2\u012b\u012c\3\2\2\2\u012c\u012d\3\2\2\2\u012d")
        buf.write("\u0135\5\36\20\2\u012e\u012f\7,\2\2\u012f\u0130\7d\2\2")
        buf.write("\u0130\u0135\5\36\20\2\u0131\u0132\7\22\2\2\u0132\u0133")
        buf.write("\7d\2\2\u0133\u0135\5\"\22\2\u0134\u0127\3\2\2\2\u0134")
        buf.write("\u012e\3\2\2\2\u0134\u0131\3\2\2\2\u0135\17\3\2\2\2\u0136")
        buf.write("\u0137\7-\2\2\u0137\u0139\7d\2\2\u0138\u013a\5\26\f\2")
        buf.write("\u0139\u0138\3\2\2\2\u0139\u013a\3\2\2\2\u013a\u013b\3")
        buf.write("\2\2\2\u013b\u013c\5\22\n\2\u013c\21\3\2\2\2\u013d\u0141")
        buf.write("\7*\2\2\u013e\u0140\5 \21\2\u013f\u013e\3\2\2\2\u0140")
        buf.write("\u0143\3\2\2\2\u0141\u013f\3\2\2\2\u0141\u0142\3\2\2\2")
        buf.write("\u0142\u0145\3\2\2\2\u0143\u0141\3\2\2\2\u0144\u0146\5")
        buf.write("\30\r\2\u0145\u0144\3\2\2\2\u0145\u0146\3\2\2\2\u0146")
        buf.write("\u014b\3\2\2\2\u0147\u014a\5> \2\u0148\u014a\5\34\17\2")
        buf.write("\u0149\u0147\3\2\2\2\u0149\u0148\3\2\2\2\u014a\u014d\3")
        buf.write("\2\2\2\u014b\u0149\3\2\2\2\u014b\u014c\3\2\2\2\u014c\u014f")
        buf.write("\3\2\2\2\u014d\u014b\3\2\2\2\u014e\u0150\5\32\16\2\u014f")
        buf.write("\u014e\3\2\2\2\u014f\u0150\3\2\2\2\u0150\u0151\3\2\2\2")
        buf.write("\u0151\u0152\7+\2\2\u0152\23\3\2\2\2\u0153\u0154\7.\2")
        buf.write("\2\u0154\u0155\7`\2\2\u0155\u0159\7\32\2\2\u0156\u015a")
        buf.write("\5\u0090I\2\u0157\u0158\7\22\2\2\u0158\u015a\5\"\22\2")
        buf.write("\u0159\u0156\3\2\2\2\u0159\u0157\3\2\2\2\u015a\u015b\3")
        buf.write("\2\2\2\u015b\u0166\7>\2\2\u015c\u015e\5\u008eH\2\u015d")
        buf.write("\u015f\5\u00acW\2\u015e\u015d\3\2\2\2\u015e\u015f\3\2")
        buf.write("\2\2\u015f\u0162\3\2\2\2\u0160\u0163\5:\36\2\u0161\u0163")
        buf.write("\7\60\2\2\u0162\u0160\3\2\2\2\u0162\u0161\3\2\2\2\u0163")
        buf.write("\u0167\3\2\2\2\u0164\u0165\7-\2\2\u0165\u0167\5\22\n\2")
        buf.write("\u0166\u015c\3\2\2\2\u0166\u0164\3\2\2\2\u0167\25\3\2")
        buf.write("\2\2\u0168\u0169\7$\2\2\u0169\u016a\5\66\34\2\u016a\27")
        buf.write("\3\2\2\2\u016b\u016c\7\32\2\2\u016c\u016d\7\33\2\2\u016d")
        buf.write("\u016e\5:\36\2\u016e\31\3\2\2\2\u016f\u0170\7\32\2\2\u0170")
        buf.write("\u0171\7\34\2\2\u0171\u0172\5:\36\2\u0172\33\3\2\2\2\u0173")
        buf.write("\u0174\7\32\2\2\u0174\u0175\7\"\2\2\u0175\u0176\5:\36")
        buf.write("\2\u0176\35\3\2\2\2\u0177\u017b\7*\2\2\u0178\u017a\5 ")
        buf.write("\21\2\u0179\u0178\3\2\2\2\u017a\u017d\3\2\2\2\u017b\u0179")
        buf.write("\3\2\2\2\u017b\u017c\3\2\2\2\u017c\u017e\3\2\2\2\u017d")
        buf.write("\u017b\3\2\2\2\u017e\u0183\7+\2\2\u017f\u0180\7$\2\2\u0180")
        buf.write("\u0183\5 \21\2\u0181\u0183\7\60\2\2\u0182\u0177\3\2\2")
        buf.write("\2\u0182\u017f\3\2\2\2\u0182\u0181\3\2\2\2\u0183\37\3")
        buf.write("\2\2\2\u0184\u0187\5*\26\2\u0185\u0187\5.\30\2\u0186\u0184")
        buf.write("\3\2\2\2\u0186\u0185\3\2\2\2\u0187!\3\2\2\2\u0188\u018b")
        buf.write("\5$\23\2\u0189\u018b\5&\24\2\u018a\u0188\3\2\2\2\u018a")
        buf.write("\u0189\3\2\2\2\u018b#\3\2\2\2\u018c\u018d\7*\2\2\u018d")
        buf.write("\u018e\5(\25\2\u018e\u018f\7\31\2\2\u018f\u0190\5:\36")
        buf.write("\2\u0190\u0191\7+\2\2\u0191\u0199\3\2\2\2\u0192\u0193")
        buf.write("\7$\2\2\u0193\u0194\5(\25\2\u0194\u0195\7\31\2\2\u0195")
        buf.write("\u0196\5:\36\2\u0196\u0197\7\60\2\2\u0197\u0199\3\2\2")
        buf.write("\2\u0198\u018c\3\2\2\2\u0198\u0192\3\2\2\2\u0199%\3\2")
        buf.write("\2\2\u019a\u019b\7*\2\2\u019b\u019c\5(\25\2\u019c\u019d")
        buf.write("\5\u00b6\\\2\u019d\u019e\7+\2\2\u019e\u01a5\3\2\2\2\u019f")
        buf.write("\u01a0\7$\2\2\u01a0\u01a1\5(\25\2\u01a1\u01a2\5\u00b6")
        buf.write("\\\2\u01a2\u01a3\7\60\2\2\u01a3\u01a5\3\2\2\2\u01a4\u019a")
        buf.write("\3\2\2\2\u01a4\u019f\3\2\2\2\u01a5\'\3\2\2\2\u01a6\u01a7")
        buf.write("\7Q\2\2\u01a7\u01a8\7P\2\2\u01a8\u01a9\7d\2\2\u01a9\u01aa")
        buf.write("\7\60\2\2\u01aa)\3\2\2\2\u01ab\u01ad\7Q\2\2\u01ac\u01ae")
        buf.write("\7R\2\2\u01ad\u01ac\3\2\2\2\u01ad\u01ae\3\2\2\2\u01ae")
        buf.write("\u01b0\3\2\2\2\u01af\u01b1\7P\2\2\u01b0\u01af\3\2\2\2")
        buf.write("\u01b0\u01b1\3\2\2\2\u01b1\u01b2\3\2\2\2\u01b2\u01b7\5")
        buf.write(",\27\2\u01b3\u01b4\7S\2\2\u01b4\u01b6\5,\27\2\u01b5\u01b3")
        buf.write("\3\2\2\2\u01b6\u01b9\3\2\2\2\u01b7\u01b5\3\2\2\2\u01b7")
        buf.write("\u01b8\3\2\2\2\u01b8\u01ba\3\2\2\2\u01b9\u01b7\3\2\2\2")
        buf.write("\u01ba\u01bb\7\60\2\2\u01bb+\3\2\2\2\u01bc\u01c1\7d\2")
        buf.write("\2\u01bd\u01be\7d\2\2\u01be\u01bf\7\61\2\2\u01bf\u01c1")
        buf.write("\5\\/\2\u01c0\u01bc\3\2\2\2\u01c0\u01bd\3\2\2\2\u01c1")
        buf.write("-\3\2\2\2\u01c2\u01c3\7T\2\2\u01c3\u01c7\5\64\33\2\u01c4")
        buf.write("\u01c5\5\62\32\2\u01c5\u01c6\5\60\31\2\u01c6\u01c8\3\2")
        buf.write("\2\2\u01c7\u01c4\3\2\2\2\u01c7\u01c8\3\2\2\2\u01c8\u01d2")
        buf.write("\3\2\2\2\u01c9\u01ca\7S\2\2\u01ca\u01ce\5\64\33\2\u01cb")
        buf.write("\u01cc\5\62\32\2\u01cc\u01cd\5\60\31\2\u01cd\u01cf\3\2")
        buf.write("\2\2\u01ce\u01cb\3\2\2\2\u01ce\u01cf\3\2\2\2\u01cf\u01d1")
        buf.write("\3\2\2\2\u01d0\u01c9\3\2\2\2\u01d1\u01d4\3\2\2\2\u01d2")
        buf.write("\u01d0\3\2\2\2\u01d2\u01d3\3\2\2\2\u01d3\u01d5\3\2\2\2")
        buf.write("\u01d4\u01d2\3\2\2\2\u01d5\u01d6\7\60\2\2\u01d6\u01de")
        buf.write("\3\2\2\2\u01d7\u01d8\7T\2\2\u01d8\u01da\7d\2\2\u01d9\u01db")
        buf.write("\5\60\31\2\u01da\u01d9\3\2\2\2\u01da\u01db\3\2\2\2\u01db")
        buf.write("\u01dc\3\2\2\2\u01dc\u01de\5:\36\2\u01dd\u01c2\3\2\2\2")
        buf.write("\u01dd\u01d7\3\2\2\2\u01de/\3\2\2\2\u01df\u01e1\7\32\2")
        buf.write("\2\u01e0\u01e2\5\66\34\2\u01e1\u01e0\3\2\2\2\u01e1\u01e2")
        buf.write("\3\2\2\2\u01e2\u01e3\3\2\2\2\u01e3\u01e4\t\2\2\2\u01e4")
        buf.write("\61\3\2\2\2\u01e5\u01e7\7%\2\2\u01e6\u01e8\58\35\2\u01e7")
        buf.write("\u01e6\3\2\2\2\u01e7\u01e8\3\2\2\2\u01e8\u01ec\3\2\2\2")
        buf.write("\u01e9\u01ed\7%\2\2\u01ea\u01eb\7)\2\2\u01eb\u01ed\5\\")
        buf.write("/\2\u01ec\u01e9\3\2\2\2\u01ec\u01ea\3\2\2\2\u01ed\63\3")
        buf.write("\2\2\2\u01ee\u01ef\7d\2\2\u01ef\u01f0\7G\2\2\u01f0\u01f1")
        buf.write("\7d\2\2\u01f1\65\3\2\2\2\u01f2\u01f7\7d\2\2\u01f3\u01f4")
        buf.write("\7S\2\2\u01f4\u01f6\7d\2\2\u01f5\u01f3\3\2\2\2\u01f6\u01f9")
        buf.write("\3\2\2\2\u01f7\u01f5\3\2\2\2\u01f7\u01f8\3\2\2\2\u01f8")
        buf.write("\67\3\2\2\2\u01f9\u01f7\3\2\2\2\u01fa\u01ff\5\\/\2\u01fb")
        buf.write("\u01fc\7S\2\2\u01fc\u01fe\5\\/\2\u01fd\u01fb\3\2\2\2\u01fe")
        buf.write("\u0201\3\2\2\2\u01ff\u01fd\3\2\2\2\u01ff\u0200\3\2\2\2")
        buf.write("\u02009\3\2\2\2\u0201\u01ff\3\2\2\2\u0202\u0206\7*\2\2")
        buf.write("\u0203\u0205\5> \2\u0204\u0203\3\2\2\2\u0205\u0208\3\2")
        buf.write("\2\2\u0206\u0204\3\2\2\2\u0206\u0207\3\2\2\2\u0207\u0209")
        buf.write("\3\2\2\2\u0208\u0206\3\2\2\2\u0209\u020d\7+\2\2\u020a")
        buf.write("\u020b\7$\2\2\u020b\u020d\5> \2\u020c\u0202\3\2\2\2\u020c")
        buf.write("\u020a\3\2\2\2\u020d;\3\2\2\2\u020e\u020f\5\66\34\2\u020f")
        buf.write("\u0210\5:\36\2\u0210=\3\2\2\2\u0211\u0224\5:\36\2\u0212")
        buf.write("\u0224\5<\37\2\u0213\u0214\5\\/\2\u0214\u0215\7\60\2\2")
        buf.write("\u0215\u0224\3\2\2\2\u0216\u0224\5@!\2\u0217\u0224\5B")
        buf.write("\"\2\u0218\u0224\5J&\2\u0219\u0224\5L\'\2\u021a\u021b")
        buf.write("\5P)\2\u021b\u021c\7\60\2\2\u021c\u0224\3\2\2\2\u021d")
        buf.write("\u021e\5N(\2\u021e\u021f\7\60\2\2\u021f\u0224\3\2\2\2")
        buf.write("\u0220\u0224\5R*\2\u0221\u0224\5T+\2\u0222\u0224\5V,\2")
        buf.write("\u0223\u0211\3\2\2\2\u0223\u0212\3\2\2\2\u0223\u0213\3")
        buf.write("\2\2\2\u0223\u0216\3\2\2\2\u0223\u0217\3\2\2\2\u0223\u0218")
        buf.write("\3\2\2\2\u0223\u0219\3\2\2\2\u0223\u021a\3\2\2\2\u0223")
        buf.write("\u021d\3\2\2\2\u0223\u0220\3\2\2\2\u0223\u0221\3\2\2\2")
        buf.write("\u0223\u0222\3\2\2\2\u0224?\3\2\2\2\u0225\u0226\79\2\2")
        buf.write("\u0226\u0227\5\\/\2\u0227\u022b\5:\36\2\u0228\u022a\5")
        buf.write("F$\2\u0229\u0228\3\2\2\2\u022a\u022d\3\2\2\2\u022b\u0229")
        buf.write("\3\2\2\2\u022b\u022c\3\2\2\2\u022c\u022f\3\2\2\2\u022d")
        buf.write("\u022b\3\2\2\2\u022e\u0230\5H%\2\u022f\u022e\3\2\2\2\u022f")
        buf.write("\u0230\3\2\2\2\u0230A\3\2\2\2\u0231\u0232\7F\2\2\u0232")
        buf.write("\u0234\5:\36\2\u0233\u0235\5D#\2\u0234\u0233\3\2\2\2\u0234")
        buf.write("\u0235\3\2\2\2\u0235C\3\2\2\2\u0236\u023a\7;\2\2\u0237")
        buf.write("\u0238\7[\2\2\u0238\u0239\7d\2\2\u0239\u023b\7\\\2\2\u023a")
        buf.write("\u0237\3\2\2\2\u023a\u023b\3\2\2\2\u023b\u023c\3\2\2\2")
        buf.write("\u023c\u0244\5:\36\2\u023d\u0240\7;\2\2\u023e\u023f\7")
        buf.write("\32\2\2\u023f\u0241\7d\2\2\u0240\u023e\3\2\2\2\u0240\u0241")
        buf.write("\3\2\2\2\u0241\u0242\3\2\2\2\u0242\u0244\5:\36\2\u0243")
        buf.write("\u0236\3\2\2\2\u0243\u023d\3\2\2\2\u0244E\3\2\2\2\u0245")
        buf.write("\u0246\7:\2\2\u0246\u0247\5\\/\2\u0247\u0248\5:\36\2\u0248")
        buf.write("G\3\2\2\2\u0249\u024a\7;\2\2\u024a\u024b\5:\36\2\u024b")
        buf.write("I\3\2\2\2\u024c\u024d\7<\2\2\u024d\u024e\5\\/\2\u024e")
        buf.write("\u024f\7=\2\2\u024f\u0250\5\\/\2\u0250\u0251\7>\2\2\u0251")
        buf.write("\u0252\5\\/\2\u0252\u0253\5:\36\2\u0253\u025b\3\2\2\2")
        buf.write("\u0254\u0255\7<\2\2\u0255\u0256\7d\2\2\u0256\u0257\7O")
        buf.write("\2\2\u0257\u0258\5\\/\2\u0258\u0259\5:\36\2\u0259\u025b")
        buf.write("\3\2\2\2\u025a\u024c\3\2\2\2\u025a\u0254\3\2\2\2\u025b")
        buf.write("K\3\2\2\2\u025c\u025d\7?\2\2\u025d\u025e\5\\/\2\u025e")
        buf.write("\u025f\5:\36\2\u025fM\3\2\2\2\u0260\u0261\t\3\2\2\u0261")
        buf.write("O\3\2\2\2\u0262\u0263\7/\2\2\u0263\u0264\5\\/\2\u0264")
        buf.write("Q\3\2\2\2\u0265\u0266\7E\2\2\u0266\u0267\5\\/\2\u0267")
        buf.write("\u0268\7\60\2\2\u0268S\3\2\2\2\u0269\u026a\7D\2\2\u026a")
        buf.write("\u026b\5\\/\2\u026b\u026c\7\60\2\2\u026cU\3\2\2\2\u026d")
        buf.write("\u0272\5X-\2\u026e\u0272\5Z.\2\u026f\u0270\7B\2\2\u0270")
        buf.write("\u0272\7\60\2\2\u0271\u026d\3\2\2\2\u0271\u026e\3\2\2")
        buf.write("\2\u0271\u026f\3\2\2\2\u0272W\3\2\2\2\u0273\u0274\7\27")
        buf.write("\2\2\u0274\u0275\5\\/\2\u0275\u0276\7\60\2\2\u0276Y\3")
        buf.write("\2\2\2\u0277\u0278\7\30\2\2\u0278\u027b\5\\/\2\u0279\u027c")
        buf.write("\7\60\2\2\u027a\u027c\5H%\2\u027b\u0279\3\2\2\2\u027b")
        buf.write("\u027a\3\2\2\2\u027c[\3\2\2\2\u027d\u0281\5d\63\2\u027e")
        buf.write("\u0282\5^\60\2\u027f\u0282\5`\61\2\u0280\u0282\5b\62\2")
        buf.write("\u0281\u027e\3\2\2\2\u0281\u027f\3\2\2\2\u0281\u0280\3")
        buf.write("\2\2\2\u0281\u0282\3\2\2\2\u0282]\3\2\2\2\u0283\u0284")
        buf.write("\7\61\2\2\u0284\u0285\5\\/\2\u0285_\3\2\2\2\u0286\u0287")
        buf.write("\7\66\2\2\u0287\u0288\5\\/\2\u0288a\3\2\2\2\u0289\u028a")
        buf.write("\t\4\2\2\u028a\u028b\5\\/\2\u028bc\3\2\2\2\u028c\u0293")
        buf.write("\5f\64\2\u028d\u028f\7H\2\2\u028e\u028d\3\2\2\2\u028e")
        buf.write("\u028f\3\2\2\2\u028f\u0290\3\2\2\2\u0290\u0291\5\u0092")
        buf.write("J\2\u0291\u0292\5\\/\2\u0292\u0294\3\2\2\2\u0293\u028e")
        buf.write("\3\2\2\2\u0293\u0294\3\2\2\2\u0294e\3\2\2\2\u0295\u029a")
        buf.write("\5h\65\2\u0296\u0297\t\5\2\2\u0297\u0299\5h\65\2\u0298")
        buf.write("\u0296\3\2\2\2\u0299\u029c\3\2\2\2\u029a\u0298\3\2\2\2")
        buf.write("\u029a\u029b\3\2\2\2\u029bg\3\2\2\2\u029c\u029a\3\2\2")
        buf.write("\2\u029d\u029e\7H\2\2\u029e\u02a9\5h\65\2\u029f\u02a5")
        buf.write("\5n8\2\u02a0\u02a1\5j\66\2\u02a1\u02a2\5n8\2\u02a2\u02a4")
        buf.write("\3\2\2\2\u02a3\u02a0\3\2\2\2\u02a4\u02a7\3\2\2\2\u02a5")
        buf.write("\u02a3\3\2\2\2\u02a5\u02a6\3\2\2\2\u02a6\u02a9\3\2\2\2")
        buf.write("\u02a7\u02a5\3\2\2\2\u02a8\u029d\3\2\2\2\u02a8\u029f\3")
        buf.write("\2\2\2\u02a9i\3\2\2\2\u02aa\u02b3\7I\2\2\u02ab\u02b3\7")
        buf.write("J\2\2\u02ac\u02b3\7K\2\2\u02ad\u02b3\7L\2\2\u02ae\u02b3")
        buf.write("\7M\2\2\u02af\u02b3\7N\2\2\u02b0\u02b3\7O\2\2\u02b1\u02b3")
        buf.write("\5l\67\2\u02b2\u02aa\3\2\2\2\u02b2\u02ab\3\2\2\2\u02b2")
        buf.write("\u02ac\3\2\2\2\u02b2\u02ad\3\2\2\2\u02b2\u02ae\3\2\2\2")
        buf.write("\u02b2\u02af\3\2\2\2\u02b2\u02b0\3\2\2\2\u02b2\u02b1\3")
        buf.write("\2\2\2\u02b3k\3\2\2\2\u02b4\u02b5\7H\2\2\u02b5\u02b6\7")
        buf.write("O\2\2\u02b6m\3\2\2\2\u02b7\u02bc\5p9\2\u02b8\u02b9\t\6")
        buf.write("\2\2\u02b9\u02bb\5p9\2\u02ba\u02b8\3\2\2\2\u02bb\u02be")
        buf.write("\3\2\2\2\u02bc\u02ba\3\2\2\2\u02bc\u02bd\3\2\2\2\u02bd")
        buf.write("o\3\2\2\2\u02be\u02bc\3\2\2\2\u02bf\u02c4\5r:\2\u02c0")
        buf.write("\u02c1\t\7\2\2\u02c1\u02c3\5r:\2\u02c2\u02c0\3\2\2\2\u02c3")
        buf.write("\u02c6\3\2\2\2\u02c4\u02c2\3\2\2\2\u02c4\u02c5\3\2\2\2")
        buf.write("\u02c5q\3\2\2\2\u02c6\u02c4\3\2\2\2\u02c7\u02c8\t\6\2")
        buf.write("\2\u02c8\u02cb\5r:\2\u02c9\u02cb\5t;\2\u02ca\u02c7\3\2")
        buf.write("\2\2\u02ca\u02c9\3\2\2\2\u02cbs\3\2\2\2\u02cc\u02d1\5")
        buf.write("v<\2\u02cd\u02ce\7Z\2\2\u02ce\u02d0\5r:\2\u02cf\u02cd")
        buf.write("\3\2\2\2\u02d0\u02d3\3\2\2\2\u02d1\u02cf\3\2\2\2\u02d1")
        buf.write("\u02d2\3\2\2\2\u02d2u\3\2\2\2\u02d3\u02d1\3\2\2\2\u02d4")
        buf.write("\u02d5\b<\1\2\u02d5\u02ec\7b\2\2\u02d6\u02ec\7_\2\2\u02d7")
        buf.write("\u02ec\7`\2\2\u02d8\u02ec\7a\2\2\u02d9\u02ec\7c\2\2\u02da")
        buf.write("\u02ec\7d\2\2\u02db\u02ec\5\u008aF\2\u02dc\u02ec\5\u009a")
        buf.write("N\2\u02dd\u02ec\5\u009eP\2\u02de\u02df\7[\2\2\u02df\u02e0")
        buf.write("\5\\/\2\u02e0\u02e1\7\\\2\2\u02e1\u02ec\3\2\2\2\u02e2")
        buf.write("\u02e3\7%\2\2\u02e3\u02e5\7d\2\2\u02e4\u02e6\5\u00acW")
        buf.write("\2\u02e5\u02e4\3\2\2\2\u02e5\u02e6\3\2\2\2\u02e6\u02ec")
        buf.write("\3\2\2\2\u02e7\u02ec\5\u00a2R\2\u02e8\u02ec\5z>\2\u02e9")
        buf.write("\u02ec\5|?\2\u02ea\u02ec\5\u00b4[\2\u02eb\u02d4\3\2\2")
        buf.write("\2\u02eb\u02d6\3\2\2\2\u02eb\u02d7\3\2\2\2\u02eb\u02d8")
        buf.write("\3\2\2\2\u02eb\u02d9\3\2\2\2\u02eb\u02da\3\2\2\2\u02eb")
        buf.write("\u02db\3\2\2\2\u02eb\u02dc\3\2\2\2\u02eb\u02dd\3\2\2\2")
        buf.write("\u02eb\u02de\3\2\2\2\u02eb\u02e2\3\2\2\2\u02eb\u02e7\3")
        buf.write("\2\2\2\u02eb\u02e8\3\2\2\2\u02eb\u02e9\3\2\2\2\u02eb\u02ea")
        buf.write("\3\2\2\2\u02ec\u02f5\3\2\2\2\u02ed\u02ef\f\7\2\2\u02ee")
        buf.write("\u02f0\5x=\2\u02ef\u02ee\3\2\2\2\u02f0\u02f1\3\2\2\2\u02f1")
        buf.write("\u02ef\3\2\2\2\u02f1\u02f2\3\2\2\2\u02f2\u02f4\3\2\2\2")
        buf.write("\u02f3\u02ed\3\2\2\2\u02f4\u02f7\3\2\2\2\u02f5\u02f3\3")
        buf.write("\2\2\2\u02f5\u02f6\3\2\2\2\u02f6w\3\2\2\2\u02f7\u02f5")
        buf.write("\3\2\2\2\u02f8\u02f9\7G\2\2\u02f9\u0308\5~@\2\u02fa\u02fb")
        buf.write("\7G\2\2\u02fb\u0308\7d\2\2\u02fc\u0308\5\u009cO\2\u02fd")
        buf.write("\u02ff\7[\2\2\u02fe\u0300\58\35\2\u02ff\u02fe\3\2\2\2")
        buf.write("\u02ff\u0300\3\2\2\2\u0300\u0301\3\2\2\2\u0301\u0308\7")
        buf.write("\\\2\2\u0302\u0303\7%\2\2\u0303\u0305\7d\2\2\u0304\u0306")
        buf.write("\5\u00acW\2\u0305\u0304\3\2\2\2\u0305\u0306\3\2\2\2\u0306")
        buf.write("\u0308\3\2\2\2\u0307\u02f8\3\2\2\2\u0307\u02fa\3\2\2\2")
        buf.write("\u0307\u02fc\3\2\2\2\u0307\u02fd\3\2\2\2\u0307\u0302\3")
        buf.write("\2\2\2\u0308y\3\2\2\2\u0309\u030a\7\4\2\2\u030a\u030b")
        buf.write("\5\\/\2\u030b{\3\2\2\2\u030c\u030d\7W\2\2\u030d\u030e")
        buf.write("\5\\/\2\u030e}\3\2\2\2\u030f\u0316\3\2\2\2\u0310\u0316")
        buf.write("\5\u0088E\2\u0311\u0316\5\u0084C\2\u0312\u0316\5\u0086")
        buf.write("D\2\u0313\u0316\5\u0082B\2\u0314\u0316\5\u0080A\2\u0315")
        buf.write("\u030f\3\2\2\2\u0315\u0310\3\2\2\2\u0315\u0311\3\2\2\2")
        buf.write("\u0315\u0312\3\2\2\2\u0315\u0313\3\2\2\2\u0315\u0314\3")
        buf.write("\2\2\2\u0316\177\3\2\2\2\u0317\u0318\5\u00b4[\2\u0318")
        buf.write("\u0081\3\2\2\2\u0319\u031a\t\b\2\2\u031a\u0083\3\2\2\2")
        buf.write("\u031b\u032d\7\36\2\2\u031c\u031d\7*\2\2\u031d\u031e\5")
        buf.write("\66\34\2\u031e\u031f\7+\2\2\u031f\u032d\3\2\2\2\u0320")
        buf.write("\u0321\7\17\2\2\u0321\u0324\7%\2\2\u0322\u0324\7(\2\2")
        buf.write("\u0323\u0320\3\2\2\2\u0323\u0322\3\2\2\2\u0324\u0325\3")
        buf.write("\2\2\2\u0325\u032a\t\t\2\2\u0326\u0327\7[\2\2\u0327\u0328")
        buf.write("\58\35\2\u0328\u0329\7\\\2\2\u0329\u032b\3\2\2\2\u032a")
        buf.write("\u0326\3\2\2\2\u032a\u032b\3\2\2\2\u032b\u032d\3\2\2\2")
        buf.write("\u032c\u031b\3\2\2\2\u032c\u031c\3\2\2\2\u032c\u0323\3")
        buf.write("\2\2\2\u032d\u0085\3\2\2\2\u032e\u033c\7\35\2\2\u032f")
        buf.write("\u0330\7\16\2\2\u0330\u0333\7%\2\2\u0331\u0333\7\'\2\2")
        buf.write("\u0332\u032f\3\2\2\2\u0332\u0331\3\2\2\2\u0333\u0334\3")
        buf.write("\2\2\2\u0334\u0339\7d\2\2\u0335\u0336\7[\2\2\u0336\u0337")
        buf.write("\58\35\2\u0337\u0338\7\\\2\2\u0338\u033a\3\2\2\2\u0339")
        buf.write("\u0335\3\2\2\2\u0339\u033a\3\2\2\2\u033a\u033c\3\2\2\2")
        buf.write("\u033b\u032e\3\2\2\2\u033b\u0332\3\2\2\2\u033c\u0087\3")
        buf.write("\2\2\2\u033d\u033e\7\13\2\2\u033e\u0341\7%\2\2\u033f\u0341")
        buf.write("\7&\2\2\u0340\u033d\3\2\2\2\u0340\u033f\3\2\2\2\u0341")
        buf.write("\u0342\3\2\2\2\u0342\u0347\7d\2\2\u0343\u0344\7[\2\2\u0344")
        buf.write("\u0345\58\35\2\u0345\u0346\7\\\2\2\u0346\u0348\3\2\2\2")
        buf.write("\u0347\u0343\3\2\2\2\u0347\u0348\3\2\2\2\u0348\u0089\3")
        buf.write("\2\2\2\u0349\u034b\5\u008cG\2\u034a\u034c\5\u00aeX\2\u034b")
        buf.write("\u034a\3\2\2\2\u034b\u034c\3\2\2\2\u034c\u0355\3\2\2\2")
        buf.write("\u034d\u0352\5\u0092J\2\u034e\u0350\5\u008cG\2\u034f\u0351")
        buf.write("\5\u00aeX\2\u0350\u034f\3\2\2\2\u0350\u0351\3\2\2\2\u0351")
        buf.write("\u0353\3\2\2\2\u0352\u034e\3\2\2\2\u0352\u0353\3\2\2\2")
        buf.write("\u0353\u0355\3\2\2\2\u0354\u0349\3\2\2\2\u0354\u034d\3")
        buf.write("\2\2\2\u0355\u008b\3\2\2\2\u0356\u0357\7\26\2\2\u0357")
        buf.write("\u0358\7%\2\2\u0358\u0359\7d\2\2\u0359\u008d\3\2\2\2\u035a")
        buf.write("\u035b\7-\2\2\u035b\u035c\7%\2\2\u035c\u035d\7d\2\2\u035d")
        buf.write("\u008f\3\2\2\2\u035e\u035f\7\22\2\2\u035f\u0360\7%\2\2")
        buf.write("\u0360\u0361\7d\2\2\u0361\u0091\3\2\2\2\u0362\u0366\5")
        buf.write("\u0094K\2\u0363\u0366\5\u0096L\2\u0364\u0366\5\u0098M")
        buf.write("\2\u0365\u0362\3\2\2\2\u0365\u0363\3\2\2\2\u0365\u0364")
        buf.write("\3\2\2\2\u0366\u0093\3\2\2\2\u0367\u0374\7\5\2\2\u0368")
        buf.write("\u0370\7V\2\2\u0369\u036a\7]\2\2\u036a\u036d\7d\2\2\u036b")
        buf.write("\u036e\5\u00acW\2\u036c\u036e\5\u00aeX\2\u036d\u036b\3")
        buf.write("\2\2\2\u036d\u036c\3\2\2\2\u036d\u036e\3\2\2\2\u036e\u036f")
        buf.write("\3\2\2\2\u036f\u0371\7^\2\2\u0370\u0369\3\2\2\2\u0370")
        buf.write("\u0371\3\2\2\2\u0371\u0372\3\2\2\2\u0372\u0374\7\6\2\2")
        buf.write("\u0373\u0367\3\2\2\2\u0373\u0368\3\2\2\2\u0374\u0095\3")
        buf.write("\2\2\2\u0375\u0382\7\7\2\2\u0376\u037e\7\b\2\2\u0377\u0378")
        buf.write("\7]\2\2\u0378\u037b\7d\2\2\u0379\u037c\5\u00acW\2\u037a")
        buf.write("\u037c\5\u00aeX\2\u037b\u0379\3\2\2\2\u037b\u037a\3\2")
        buf.write("\2\2\u037b\u037c\3\2\2\2\u037c\u037d\3\2\2\2\u037d\u037f")
        buf.write("\7^\2\2\u037e\u0377\3\2\2\2\u037e\u037f\3\2\2\2\u037f")
        buf.write("\u0380\3\2\2\2\u0380\u0382\7V\2\2\u0381\u0375\3\2\2\2")
        buf.write("\u0381\u0376\3\2\2\2\u0382\u0097\3\2\2\2\u0383\u0390\7")
        buf.write("\t\2\2\u0384\u038c\7\b\2\2\u0385\u0386\7]\2\2\u0386\u0389")
        buf.write("\7d\2\2\u0387\u038a\5\u00acW\2\u0388\u038a\5\u00aeX\2")
        buf.write("\u0389\u0387\3\2\2\2\u0389\u0388\3\2\2\2\u0389\u038a\3")
        buf.write("\2\2\2\u038a\u038b\3\2\2\2\u038b\u038d\7^\2\2\u038c\u0385")
        buf.write("\3\2\2\2\u038c\u038d\3\2\2\2\u038d\u038e\3\2\2\2\u038e")
        buf.write("\u0390\7\6\2\2\u038f\u0383\3\2\2\2\u038f\u0384\3\2\2\2")
        buf.write("\u0390\u0099\3\2\2\2\u0391\u0393\7]\2\2\u0392\u0394\5")
        buf.write("8\35\2\u0393\u0392\3\2\2\2\u0393\u0394\3\2\2\2\u0394\u0395")
        buf.write("\3\2\2\2\u0395\u0396\7^\2\2\u0396\u009b\3\2\2\2\u0397")
        buf.write("\u0398\7]\2\2\u0398\u0399\5\\/\2\u0399\u039a\7^\2\2\u039a")
        buf.write("\u03a2\3\2\2\2\u039b\u039c\7]\2\2\u039c\u039d\5\\/\2\u039d")
        buf.write("\u039e\7$\2\2\u039e\u039f\5\\/\2\u039f\u03a0\7^\2\2\u03a0")
        buf.write("\u03a2\3\2\2\2\u03a1\u0397\3\2\2\2\u03a1\u039b\3\2\2\2")
        buf.write("\u03a2\u009d\3\2\2\2\u03a3\u03ac\7*\2\2\u03a4\u03a9\5")
        buf.write("\u00a0Q\2\u03a5\u03a6\7S\2\2\u03a6\u03a8\5\u00a0Q\2\u03a7")
        buf.write("\u03a5\3\2\2\2\u03a8\u03ab\3\2\2\2\u03a9\u03a7\3\2\2\2")
        buf.write("\u03a9\u03aa\3\2\2\2\u03aa\u03ad\3\2\2\2\u03ab\u03a9\3")
        buf.write("\2\2\2\u03ac\u03a4\3\2\2\2\u03ac\u03ad\3\2\2\2\u03ad\u03ae")
        buf.write("\3\2\2\2\u03ae\u03af\7+\2\2\u03af\u009f\3\2\2\2\u03b0")
        buf.write("\u03b1\7`\2\2\u03b1\u03b2\7$\2\2\u03b2\u03b3\5\\/\2\u03b3")
        buf.write("\u00a1\3\2\2\2\u03b4\u03b6\7\31\2\2\u03b5\u03b7\5\\/\2")
        buf.write("\u03b6\u03b5\3\2\2\2\u03b6\u03b7\3\2\2\2\u03b7\u03b8\3")
        buf.write("\2\2\2\u03b8\u03b9\5\u00a4S\2\u03b9\u00a3\3\2\2\2\u03ba")
        buf.write("\u03be\5\u00a6T\2\u03bb\u03be\5\u00aaV\2\u03bc\u03be\5")
        buf.write("\u00a8U\2\u03bd\u03ba\3\2\2\2\u03bd\u03bb\3\2\2\2\u03bd")
        buf.write("\u03bc\3\2\2\2\u03be\u00a5\3\2\2\2\u03bf\u03c1\5\u0092")
        buf.write("J\2\u03c0\u03bf\3\2\2\2\u03c0\u03c1\3\2\2\2\u03c1\u03c2")
        buf.write("\3\2\2\2\u03c2\u03c4\5\u008cG\2\u03c3\u03c5\5\u00acW\2")
        buf.write("\u03c4\u03c3\3\2\2\2\u03c4\u03c5\3\2\2\2\u03c5\u00a7\3")
        buf.write("\2\2\2\u03c6\u03c7\5\u0092J\2\u03c7\u03c8\5\u0090I\2\u03c8")
        buf.write("\u00a9\3\2\2\2\u03c9\u03cb\5\u008eH\2\u03ca\u03cc\5\u00ac")
        buf.write("W\2\u03cb\u03ca\3\2\2\2\u03cb\u03cc\3\2\2\2\u03cc\u00ab")
        buf.write("\3\2\2\2\u03cd\u03d6\7[\2\2\u03ce\u03d3\5\u00b0Y\2\u03cf")
        buf.write("\u03d0\7S\2\2\u03d0\u03d2\5\u00b0Y\2\u03d1\u03cf\3\2\2")
        buf.write("\2\u03d2\u03d5\3\2\2\2\u03d3\u03d1\3\2\2\2\u03d3\u03d4")
        buf.write("\3\2\2\2\u03d4\u03d7\3\2\2\2\u03d5\u03d3\3\2\2\2\u03d6")
        buf.write("\u03ce\3\2\2\2\u03d6\u03d7\3\2\2\2\u03d7\u03d8\3\2\2\2")
        buf.write("\u03d8\u03d9\7\\\2\2\u03d9\u00ad\3\2\2\2\u03da\u03e3\7")
        buf.write("[\2\2\u03db\u03e0\5\u00b2Z\2\u03dc\u03dd\7S\2\2\u03dd")
        buf.write("\u03df\5\u00b2Z\2\u03de\u03dc\3\2\2\2\u03df\u03e2\3\2")
        buf.write("\2\2\u03e0\u03de\3\2\2\2\u03e0\u03e1\3\2\2\2\u03e1\u03e4")
        buf.write("\3\2\2\2\u03e2\u03e0\3\2\2\2\u03e3\u03db\3\2\2\2\u03e3")
        buf.write("\u03e4\3\2\2\2\u03e4\u03e5\3\2\2\2\u03e5\u03e6\7\\\2\2")
        buf.write("\u03e6\u00af\3\2\2\2\u03e7\u03e8\7d\2\2\u03e8\u03e9\7")
        buf.write("\61\2\2\u03e9\u03ea\5\\/\2\u03ea\u00b1\3\2\2\2\u03eb\u03ec")
        buf.write("\7d\2\2\u03ec\u03ed\5j\66\2\u03ed\u03ee\5\\/\2\u03ee\u00b3")
        buf.write("\3\2\2\2\u03ef\u03f0\t\n\2\2\u03f0\u00b5\3\2\2\2\u03f1")
        buf.write("\u03f3\7\23\2\2\u03f2\u03f1\3\2\2\2\u03f2\u03f3\3\2\2")
        buf.write("\2\u03f3\u03f4\3\2\2\2\u03f4\u03f6\t\13\2\2\u03f5\u03f7")
        buf.write("\5\u00d0i\2\u03f6\u03f5\3\2\2\2\u03f6\u03f7\3\2\2\2\u03f7")
        buf.write("\u03f8\3\2\2\2\u03f8\u03f9\7*\2\2\u03f9\u03fa\5\u00b8")
        buf.write("]\2\u03fa\u03fb\7+\2\2\u03fb\u00b7\3\2\2\2\u03fc\u03fe")
        buf.write("\5\u00ba^\2\u03fd\u03ff\7\60\2\2\u03fe\u03fd\3\2\2\2\u03fe")
        buf.write("\u03ff\3\2\2\2\u03ff\u0401\3\2\2\2\u0400\u03fc\3\2\2\2")
        buf.write("\u0401\u0404\3\2\2\2\u0402\u0400\3\2\2\2\u0402\u0403\3")
        buf.write("\2\2\2\u0403\u00b9\3\2\2\2\u0404\u0402\3\2\2\2\u0405\u040e")
        buf.write("\5\u00c8e\2\u0406\u040e\5\u00c2b\2\u0407\u040e\5\u00bc")
        buf.write("_\2\u0408\u0409\5\u00d0i\2\u0409\u040a\7\61\2\2\u040a")
        buf.write("\u040b\5\u00d0i\2\u040b\u040e\3\2\2\2\u040c\u040e\5\u00ce")
        buf.write("h\2\u040d\u0405\3\2\2\2\u040d\u0406\3\2\2\2\u040d\u0407")
        buf.write("\3\2\2\2\u040d\u0408\3\2\2\2\u040d\u040c\3\2\2\2\u040e")
        buf.write("\u00bb\3\2\2\2\u040f\u0410\t\f\2\2\u0410\u0411\5\u00be")
        buf.write("`\2\u0411\u00bd\3\2\2\2\u0412\u0414\7]\2\2\u0413\u0415")
        buf.write("\5\u00c0a\2\u0414\u0413\3\2\2\2\u0414\u0415\3\2\2\2\u0415")
        buf.write("\u0416\3\2\2\2\u0416\u0418\7^\2\2\u0417\u0412\3\2\2\2")
        buf.write("\u0418\u0419\3\2\2\2\u0419\u0417\3\2\2\2\u0419\u041a\3")
        buf.write("\2\2\2\u041a\u00bf\3\2\2\2\u041b\u041e\5\u00d0i\2\u041c")
        buf.write("\u041d\7\61\2\2\u041d\u041f\5\u00d0i\2\u041e\u041c\3\2")
        buf.write("\2\2\u041e\u041f\3\2\2\2\u041f\u0421\3\2\2\2\u0420\u0422")
        buf.write("\7S\2\2\u0421\u0420\3\2\2\2\u0421\u0422\3\2\2\2\u0422")
        buf.write("\u0424\3\2\2\2\u0423\u041b\3\2\2\2\u0424\u0425\3\2\2\2")
        buf.write("\u0425\u0423\3\2\2\2\u0425\u0426\3\2\2\2\u0426\u00c1\3")
        buf.write("\2\2\2\u0427\u042a\5\u00caf\2\u0428\u042a\5\u00ceh\2\u0429")
        buf.write("\u0427\3\2\2\2\u0429\u0428\3\2\2\2\u042a\u042b\3\2\2\2")
        buf.write("\u042b\u042d\5\u00c4c\2\u042c\u042e\5\u00be`\2\u042d\u042c")
        buf.write("\3\2\2\2\u042d\u042e\3\2\2\2\u042e\u00c3\3\2\2\2\u042f")
        buf.write("\u0432\5\u00c6d\2\u0430\u0433\5\u00caf\2\u0431\u0433\5")
        buf.write("\u00ceh\2\u0432\u0430\3\2\2\2\u0432\u0431\3\2\2\2\u0433")
        buf.write("\u0435\3\2\2\2\u0434\u042f\3\2\2\2\u0435\u0436\3\2\2\2")
        buf.write("\u0436\u0434\3\2\2\2\u0436\u0437\3\2\2\2\u0437\u00c5\3")
        buf.write("\2\2\2\u0438\u0439\t\r\2\2\u0439\u00c7\3\2\2\2\u043a\u043c")
        buf.write("\5\u00caf\2\u043b\u043d\5\u00be`\2\u043c\u043b\3\2\2\2")
        buf.write("\u043c\u043d\3\2\2\2\u043d\u00c9\3\2\2\2\u043e\u0440\5")
        buf.write("\u00d0i\2\u043f\u0441\5\u00ccg\2\u0440\u043f\3\2\2\2\u0440")
        buf.write("\u0441\3\2\2\2\u0441\u00cb\3\2\2\2\u0442\u0443\7$\2\2")
        buf.write("\u0443\u0446\5\u00d0i\2\u0444\u0445\7$\2\2\u0445\u0447")
        buf.write("\5\u00d0i\2\u0446\u0444\3\2\2\2\u0446\u0447\3\2\2\2\u0447")
        buf.write("\u00cd\3\2\2\2\u0448\u044a\7\25\2\2\u0449\u044b\5\u00d0")
        buf.write("i\2\u044a\u0449\3\2\2\2\u044a\u044b\3\2\2\2\u044b\u044d")
        buf.write("\3\2\2\2\u044c\u0448\3\2\2\2\u044c\u044d\3\2\2\2\u044d")
        buf.write("\u044e\3\2\2\2\u044e\u044f\7*\2\2\u044f\u0450\5\u00b8")
        buf.write("]\2\u0450\u0451\7+\2\2\u0451\u00cf\3\2\2\2\u0452\u0453")
        buf.write("\t\16\2\2\u0453\u00d1\3\2\2\2\u0087\u00d3\u00d8\u00de")
        buf.write("\u00e6\u00f1\u00f6\u00fa\u00ff\u0103\u0108\u010c\u0111")
        buf.write("\u0115\u0117\u0120\u0125\u012b\u0134\u0139\u0141\u0145")
        buf.write("\u0149\u014b\u014f\u0159\u015e\u0162\u0166\u017b\u0182")
        buf.write("\u0186\u018a\u0198\u01a4\u01ad\u01b0\u01b7\u01c0\u01c7")
        buf.write("\u01ce\u01d2\u01da\u01dd\u01e1\u01e7\u01ec\u01f7\u01ff")
        buf.write("\u0206\u020c\u0223\u022b\u022f\u0234\u023a\u0240\u0243")
        buf.write("\u025a\u0271\u027b\u0281\u028e\u0293\u029a\u02a5\u02a8")
        buf.write("\u02b2\u02bc\u02c4\u02ca\u02d1\u02e5\u02eb\u02f1\u02f5")
        buf.write("\u02ff\u0305\u0307\u0315\u0323\u032a\u032c\u0332\u0339")
        buf.write("\u033b\u0340\u0347\u034b\u0350\u0352\u0354\u0365\u036d")
        buf.write("\u0370\u0373\u037b\u037e\u0381\u0389\u038c\u038f\u0393")
        buf.write("\u03a1\u03a9\u03ac\u03b6\u03bd\u03c0\u03c4\u03cb\u03d3")
        buf.write("\u03d6\u03e0\u03e3\u03f2\u03f6\u03fe\u0402\u040d\u0414")
        buf.write("\u0419\u041e\u0421\u0425\u0429\u042d\u0432\u0436\u043c")
        buf.write("\u0440\u0446\u044a\u044c")
        return buf.getvalue()


class jacParser ( Parser ):

    grammarFileName = "jac.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'version'", "'&'", "'-->'", "'->'", "'<--'", 
                     "'<-'", "'<-->'", "'--'", "'str'", "'int'", "'float'", 
                     "'list'", "'dict'", "'bool'", "'type'", "'graph'", 
                     "'strict'", "'digraph'", "'subgraph'", "'node'", "'ignore'", 
                     "'take'", "'spawn'", "'with'", "'entry'", "'exit'", 
                     "'length'", "'keys'", "'context'", "'info'", "'details'", 
                     "'activity'", "'import'", "':'", "'::'", "'s::'", "'l::'", 
                     "'d::'", "'::>'", "'{'", "'}'", "'edge'", "'walker'", 
                     "'test'", "'assert'", "';'", "'='", "'+='", "'-='", 
                     "'*='", "'/='", "':='", "<INVALID>", "<INVALID>", "'if'", 
                     "'elif'", "'else'", "'for'", "'to'", "'by'", "'while'", 
                     "'continue'", "'break'", "'disengage'", "'skip'", "'report'", 
                     "'destroy'", "'try'", "'.'", "<INVALID>", "'=='", "'<'", 
                     "'>'", "'<='", "'>='", "'!='", "'in'", "'anchor'", 
                     "'has'", "'private'", "','", "'can'", "'+'", "'-'", 
                     "'*'", "'/'", "'%'", "'^'", "'('", "')'", "'['", "']'", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "'null'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "TYP_STRING", "TYP_INT", "TYP_FLOAT", 
                      "TYP_LIST", "TYP_DICT", "TYP_BOOL", "KW_TYPE", "KW_GRAPH", 
                      "KW_STRICT", "KW_DIGRAPH", "KW_SUBGRAPH", "KW_NODE", 
                      "KW_IGNORE", "KW_TAKE", "KW_SPAWN", "KW_WITH", "KW_ENTRY", 
                      "KW_EXIT", "KW_LENGTH", "KW_KEYS", "KW_CONTEXT", "KW_INFO", 
                      "KW_DETAILS", "KW_ACTIVITY", "KW_IMPORT", "COLON", 
                      "DBL_COLON", "STR_DBL_COLON", "LIST_DBL_COLON", "DICT_DBL_COLON", 
                      "COLON_OUT", "LBRACE", "RBRACE", "KW_EDGE", "KW_WALKER", 
                      "KW_TEST", "KW_ASSERT", "SEMI", "EQ", "PEQ", "MEQ", 
                      "TEQ", "DEQ", "CPY_EQ", "KW_AND", "KW_OR", "KW_IF", 
                      "KW_ELIF", "KW_ELSE", "KW_FOR", "KW_TO", "KW_BY", 
                      "KW_WHILE", "KW_CONTINUE", "KW_BREAK", "KW_DISENGAGE", 
                      "KW_SKIP", "KW_REPORT", "KW_DESTROY", "KW_TRY", "DOT", 
                      "NOT", "EE", "LT", "GT", "LTE", "GTE", "NE", "KW_IN", 
                      "KW_ANCHOR", "KW_HAS", "KW_PRIVATE", "COMMA", "KW_CAN", 
                      "PLUS", "MINUS", "STAR_MUL", "DIV", "MOD", "POW", 
                      "LPAREN", "RPAREN", "LSQUARE", "RSQUARE", "FLOAT", 
                      "STRING", "BOOL", "INT", "NULL", "NAME", "COMMENT", 
                      "LINE_COMMENT", "PY_COMMENT", "WS", "ErrorChar" ]

    RULE_start = 0
    RULE_import_module = 1
    RULE_ver_label = 2
    RULE_import_items = 3
    RULE_import_names = 4
    RULE_element = 5
    RULE_architype = 6
    RULE_walker = 7
    RULE_walker_block = 8
    RULE_test = 9
    RULE_namespaces = 10
    RULE_walk_entry_block = 11
    RULE_walk_exit_block = 12
    RULE_walk_activity_block = 13
    RULE_attr_block = 14
    RULE_attr_stmt = 15
    RULE_graph_block = 16
    RULE_graph_block_spawn = 17
    RULE_graph_block_dot = 18
    RULE_has_root = 19
    RULE_has_stmt = 20
    RULE_has_assign = 21
    RULE_can_stmt = 22
    RULE_event_clause = 23
    RULE_preset_in_out = 24
    RULE_dotted_name = 25
    RULE_name_list = 26
    RULE_expr_list = 27
    RULE_code_block = 28
    RULE_node_ctx_block = 29
    RULE_statement = 30
    RULE_if_stmt = 31
    RULE_try_stmt = 32
    RULE_else_from_try = 33
    RULE_elif_stmt = 34
    RULE_else_stmt = 35
    RULE_for_stmt = 36
    RULE_while_stmt = 37
    RULE_ctrl_stmt = 38
    RULE_assert_stmt = 39
    RULE_destroy_action = 40
    RULE_report_action = 41
    RULE_walker_action = 42
    RULE_ignore_action = 43
    RULE_take_action = 44
    RULE_expression = 45
    RULE_assignment = 46
    RULE_copy_assign = 47
    RULE_inc_assign = 48
    RULE_connect = 49
    RULE_logical = 50
    RULE_compare = 51
    RULE_cmp_op = 52
    RULE_nin = 53
    RULE_arithmetic = 54
    RULE_term = 55
    RULE_factor = 56
    RULE_power = 57
    RULE_atom = 58
    RULE_atom_trailer = 59
    RULE_ref = 60
    RULE_deref = 61
    RULE_built_in = 62
    RULE_cast_built_in = 63
    RULE_obj_built_in = 64
    RULE_dict_built_in = 65
    RULE_list_built_in = 66
    RULE_string_built_in = 67
    RULE_node_edge_ref = 68
    RULE_node_ref = 69
    RULE_walker_ref = 70
    RULE_graph_ref = 71
    RULE_edge_ref = 72
    RULE_edge_to = 73
    RULE_edge_from = 74
    RULE_edge_any = 75
    RULE_list_val = 76
    RULE_index_slice = 77
    RULE_dict_val = 78
    RULE_kv_pair = 79
    RULE_spawn = 80
    RULE_spawn_object = 81
    RULE_node_spawn = 82
    RULE_graph_spawn = 83
    RULE_walker_spawn = 84
    RULE_spawn_ctx = 85
    RULE_filter_ctx = 86
    RULE_spawn_assign = 87
    RULE_filter_compare = 88
    RULE_any_type = 89
    RULE_dot_graph = 90
    RULE_dot_stmt_list = 91
    RULE_dot_stmt = 92
    RULE_dot_attr_stmt = 93
    RULE_dot_attr_list = 94
    RULE_dot_a_list = 95
    RULE_dot_edge_stmt = 96
    RULE_dot_edgeRHS = 97
    RULE_dot_edgeop = 98
    RULE_dot_node_stmt = 99
    RULE_dot_node_id = 100
    RULE_dot_port = 101
    RULE_dot_subgraph = 102
    RULE_dot_id = 103

    ruleNames =  [ "start", "import_module", "ver_label", "import_items", 
                   "import_names", "element", "architype", "walker", "walker_block", 
                   "test", "namespaces", "walk_entry_block", "walk_exit_block", 
                   "walk_activity_block", "attr_block", "attr_stmt", "graph_block", 
                   "graph_block_spawn", "graph_block_dot", "has_root", "has_stmt", 
                   "has_assign", "can_stmt", "event_clause", "preset_in_out", 
                   "dotted_name", "name_list", "expr_list", "code_block", 
                   "node_ctx_block", "statement", "if_stmt", "try_stmt", 
                   "else_from_try", "elif_stmt", "else_stmt", "for_stmt", 
                   "while_stmt", "ctrl_stmt", "assert_stmt", "destroy_action", 
                   "report_action", "walker_action", "ignore_action", "take_action", 
                   "expression", "assignment", "copy_assign", "inc_assign", 
                   "connect", "logical", "compare", "cmp_op", "nin", "arithmetic", 
                   "term", "factor", "power", "atom", "atom_trailer", "ref", 
                   "deref", "built_in", "cast_built_in", "obj_built_in", 
                   "dict_built_in", "list_built_in", "string_built_in", 
                   "node_edge_ref", "node_ref", "walker_ref", "graph_ref", 
                   "edge_ref", "edge_to", "edge_from", "edge_any", "list_val", 
                   "index_slice", "dict_val", "kv_pair", "spawn", "spawn_object", 
                   "node_spawn", "graph_spawn", "walker_spawn", "spawn_ctx", 
                   "filter_ctx", "spawn_assign", "filter_compare", "any_type", 
                   "dot_graph", "dot_stmt_list", "dot_stmt", "dot_attr_stmt", 
                   "dot_attr_list", "dot_a_list", "dot_edge_stmt", "dot_edgeRHS", 
                   "dot_edgeop", "dot_node_stmt", "dot_node_id", "dot_port", 
                   "dot_subgraph", "dot_id" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    TYP_STRING=9
    TYP_INT=10
    TYP_FLOAT=11
    TYP_LIST=12
    TYP_DICT=13
    TYP_BOOL=14
    KW_TYPE=15
    KW_GRAPH=16
    KW_STRICT=17
    KW_DIGRAPH=18
    KW_SUBGRAPH=19
    KW_NODE=20
    KW_IGNORE=21
    KW_TAKE=22
    KW_SPAWN=23
    KW_WITH=24
    KW_ENTRY=25
    KW_EXIT=26
    KW_LENGTH=27
    KW_KEYS=28
    KW_CONTEXT=29
    KW_INFO=30
    KW_DETAILS=31
    KW_ACTIVITY=32
    KW_IMPORT=33
    COLON=34
    DBL_COLON=35
    STR_DBL_COLON=36
    LIST_DBL_COLON=37
    DICT_DBL_COLON=38
    COLON_OUT=39
    LBRACE=40
    RBRACE=41
    KW_EDGE=42
    KW_WALKER=43
    KW_TEST=44
    KW_ASSERT=45
    SEMI=46
    EQ=47
    PEQ=48
    MEQ=49
    TEQ=50
    DEQ=51
    CPY_EQ=52
    KW_AND=53
    KW_OR=54
    KW_IF=55
    KW_ELIF=56
    KW_ELSE=57
    KW_FOR=58
    KW_TO=59
    KW_BY=60
    KW_WHILE=61
    KW_CONTINUE=62
    KW_BREAK=63
    KW_DISENGAGE=64
    KW_SKIP=65
    KW_REPORT=66
    KW_DESTROY=67
    KW_TRY=68
    DOT=69
    NOT=70
    EE=71
    LT=72
    GT=73
    LTE=74
    GTE=75
    NE=76
    KW_IN=77
    KW_ANCHOR=78
    KW_HAS=79
    KW_PRIVATE=80
    COMMA=81
    KW_CAN=82
    PLUS=83
    MINUS=84
    STAR_MUL=85
    DIV=86
    MOD=87
    POW=88
    LPAREN=89
    RPAREN=90
    LSQUARE=91
    RSQUARE=92
    FLOAT=93
    STRING=94
    BOOL=95
    INT=96
    NULL=97
    NAME=98
    COMMENT=99
    LINE_COMMENT=100
    PY_COMMENT=101
    WS=102
    ErrorChar=103

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.9.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class StartContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(jacParser.EOF, 0)

        def ver_label(self):
            return self.getTypedRuleContext(jacParser.Ver_labelContext,0)


        def import_module(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Import_moduleContext)
            else:
                return self.getTypedRuleContext(jacParser.Import_moduleContext,i)


        def element(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.ElementContext)
            else:
                return self.getTypedRuleContext(jacParser.ElementContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_start

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStart" ):
                listener.enterStart(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStart" ):
                listener.exitStart(self)




    def start(self):

        localctx = jacParser.StartContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_start)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 209
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.T__0:
                self.state = 208
                self.ver_label()


            self.state = 214
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==jacParser.KW_IMPORT:
                self.state = 211
                self.import_module()
                self.state = 216
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 218 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 217
                self.element()
                self.state = 220 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE) | (1 << jacParser.KW_WALKER) | (1 << jacParser.KW_TEST))) != 0)):
                    break

            self.state = 222
            self.match(jacParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Import_moduleContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_IMPORT(self):
            return self.getToken(jacParser.KW_IMPORT, 0)

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def import_items(self):
            return self.getTypedRuleContext(jacParser.Import_itemsContext,0)


        def STAR_MUL(self):
            return self.getToken(jacParser.STAR_MUL, 0)

        def getRuleIndex(self):
            return jacParser.RULE_import_module

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterImport_module" ):
                listener.enterImport_module(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitImport_module" ):
                listener.exitImport_module(self)




    def import_module(self):

        localctx = jacParser.Import_moduleContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_import_module)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 224
            self.match(jacParser.KW_IMPORT)
            self.state = 225
            self.match(jacParser.LBRACE)
            self.state = 228
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_GRAPH, jacParser.KW_NODE, jacParser.KW_EDGE, jacParser.KW_WALKER]:
                self.state = 226
                self.import_items()
                pass
            elif token in [jacParser.STAR_MUL]:
                self.state = 227
                self.match(jacParser.STAR_MUL)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 230
            self.match(jacParser.RBRACE)
            self.state = 231
            self.match(jacParser.KW_WITH)
            self.state = 232
            self.match(jacParser.STRING)
            self.state = 233
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Ver_labelContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_ver_label

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterVer_label" ):
                listener.enterVer_label(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitVer_label" ):
                listener.exitVer_label(self)




    def ver_label(self):

        localctx = jacParser.Ver_labelContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_ver_label)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 235
            self.match(jacParser.T__0)
            self.state = 236
            self.match(jacParser.COLON)
            self.state = 237
            self.match(jacParser.STRING)
            self.state = 239
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.SEMI:
                self.state = 238
                self.match(jacParser.SEMI)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Import_itemsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WALKER(self):
            return self.getToken(jacParser.KW_WALKER, 0)

        def STAR_MUL(self):
            return self.getToken(jacParser.STAR_MUL, 0)

        def import_names(self):
            return self.getTypedRuleContext(jacParser.Import_namesContext,0)


        def COMMA(self):
            return self.getToken(jacParser.COMMA, 0)

        def import_items(self):
            return self.getTypedRuleContext(jacParser.Import_itemsContext,0)


        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def KW_EDGE(self):
            return self.getToken(jacParser.KW_EDGE, 0)

        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def getRuleIndex(self):
            return jacParser.RULE_import_items

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterImport_items" ):
                listener.enterImport_items(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitImport_items" ):
                listener.exitImport_items(self)




    def import_items(self):

        localctx = jacParser.Import_itemsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_import_items)
        self._la = 0 # Token type
        try:
            self.state = 277
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_WALKER]:
                self.enterOuterAlt(localctx, 1)
                self.state = 241
                self.match(jacParser.KW_WALKER)
                self.state = 244
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.STAR_MUL]:
                    self.state = 242
                    self.match(jacParser.STAR_MUL)
                    pass
                elif token in [jacParser.DBL_COLON]:
                    self.state = 243
                    self.import_names()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 248
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.COMMA:
                    self.state = 246
                    self.match(jacParser.COMMA)
                    self.state = 247
                    self.import_items()


                pass
            elif token in [jacParser.KW_NODE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 250
                self.match(jacParser.KW_NODE)
                self.state = 253
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.STAR_MUL]:
                    self.state = 251
                    self.match(jacParser.STAR_MUL)
                    pass
                elif token in [jacParser.DBL_COLON]:
                    self.state = 252
                    self.import_names()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 257
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.COMMA:
                    self.state = 255
                    self.match(jacParser.COMMA)
                    self.state = 256
                    self.import_items()


                pass
            elif token in [jacParser.KW_EDGE]:
                self.enterOuterAlt(localctx, 3)
                self.state = 259
                self.match(jacParser.KW_EDGE)
                self.state = 262
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.STAR_MUL]:
                    self.state = 260
                    self.match(jacParser.STAR_MUL)
                    pass
                elif token in [jacParser.DBL_COLON]:
                    self.state = 261
                    self.import_names()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 266
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.COMMA:
                    self.state = 264
                    self.match(jacParser.COMMA)
                    self.state = 265
                    self.import_items()


                pass
            elif token in [jacParser.KW_GRAPH]:
                self.enterOuterAlt(localctx, 4)
                self.state = 268
                self.match(jacParser.KW_GRAPH)
                self.state = 271
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.STAR_MUL]:
                    self.state = 269
                    self.match(jacParser.STAR_MUL)
                    pass
                elif token in [jacParser.DBL_COLON]:
                    self.state = 270
                    self.import_names()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 275
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.COMMA:
                    self.state = 273
                    self.match(jacParser.COMMA)
                    self.state = 274
                    self.import_items()


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Import_namesContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def name_list(self):
            return self.getTypedRuleContext(jacParser.Name_listContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def getRuleIndex(self):
            return jacParser.RULE_import_names

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterImport_names" ):
                listener.enterImport_names(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitImport_names" ):
                listener.exitImport_names(self)




    def import_names(self):

        localctx = jacParser.Import_namesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_import_names)
        try:
            self.state = 286
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,14,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 279
                self.match(jacParser.DBL_COLON)
                self.state = 280
                self.match(jacParser.NAME)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 281
                self.match(jacParser.DBL_COLON)
                self.state = 282
                self.match(jacParser.LBRACE)
                self.state = 283
                self.name_list()
                self.state = 284
                self.match(jacParser.RBRACE)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ElementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def architype(self):
            return self.getTypedRuleContext(jacParser.ArchitypeContext,0)


        def walker(self):
            return self.getTypedRuleContext(jacParser.WalkerContext,0)


        def test(self):
            return self.getTypedRuleContext(jacParser.TestContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_element

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterElement" ):
                listener.enterElement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitElement" ):
                listener.exitElement(self)




    def element(self):

        localctx = jacParser.ElementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_element)
        try:
            self.state = 291
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_GRAPH, jacParser.KW_NODE, jacParser.KW_EDGE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 288
                self.architype()
                pass
            elif token in [jacParser.KW_WALKER]:
                self.enterOuterAlt(localctx, 2)
                self.state = 289
                self.walker()
                pass
            elif token in [jacParser.KW_TEST]:
                self.enterOuterAlt(localctx, 3)
                self.state = 290
                self.test()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArchitypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def attr_block(self):
            return self.getTypedRuleContext(jacParser.Attr_blockContext,0)


        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def INT(self):
            return self.getToken(jacParser.INT, 0)

        def KW_EDGE(self):
            return self.getToken(jacParser.KW_EDGE, 0)

        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def graph_block(self):
            return self.getTypedRuleContext(jacParser.Graph_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_architype

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitype" ):
                listener.enterArchitype(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitype" ):
                listener.exitArchitype(self)




    def architype(self):

        localctx = jacParser.ArchitypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_architype)
        try:
            self.state = 306
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_NODE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 293
                self.match(jacParser.KW_NODE)
                self.state = 294
                self.match(jacParser.NAME)
                self.state = 297
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,16,self._ctx)
                if la_ == 1:
                    self.state = 295
                    self.match(jacParser.COLON)
                    self.state = 296
                    self.match(jacParser.INT)


                self.state = 299
                self.attr_block()
                pass
            elif token in [jacParser.KW_EDGE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 300
                self.match(jacParser.KW_EDGE)
                self.state = 301
                self.match(jacParser.NAME)
                self.state = 302
                self.attr_block()
                pass
            elif token in [jacParser.KW_GRAPH]:
                self.enterOuterAlt(localctx, 3)
                self.state = 303
                self.match(jacParser.KW_GRAPH)
                self.state = 304
                self.match(jacParser.NAME)
                self.state = 305
                self.graph_block()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WalkerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WALKER(self):
            return self.getToken(jacParser.KW_WALKER, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def walker_block(self):
            return self.getTypedRuleContext(jacParser.Walker_blockContext,0)


        def namespaces(self):
            return self.getTypedRuleContext(jacParser.NamespacesContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walker

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalker" ):
                listener.enterWalker(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalker" ):
                listener.exitWalker(self)




    def walker(self):

        localctx = jacParser.WalkerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_walker)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 308
            self.match(jacParser.KW_WALKER)
            self.state = 309
            self.match(jacParser.NAME)
            self.state = 311
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.COLON:
                self.state = 310
                self.namespaces()


            self.state = 313
            self.walker_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walker_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def attr_stmt(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Attr_stmtContext)
            else:
                return self.getTypedRuleContext(jacParser.Attr_stmtContext,i)


        def walk_entry_block(self):
            return self.getTypedRuleContext(jacParser.Walk_entry_blockContext,0)


        def statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.StatementContext)
            else:
                return self.getTypedRuleContext(jacParser.StatementContext,i)


        def walk_activity_block(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Walk_activity_blockContext)
            else:
                return self.getTypedRuleContext(jacParser.Walk_activity_blockContext,i)


        def walk_exit_block(self):
            return self.getTypedRuleContext(jacParser.Walk_exit_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walker_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalker_block" ):
                listener.enterWalker_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalker_block" ):
                listener.exitWalker_block(self)




    def walker_block(self):

        localctx = jacParser.Walker_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_walker_block)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 315
            self.match(jacParser.LBRACE)
            self.state = 319
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==jacParser.KW_HAS or _la==jacParser.KW_CAN:
                self.state = 316
                self.attr_stmt()
                self.state = 321
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 323
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,20,self._ctx)
            if la_ == 1:
                self.state = 322
                self.walk_entry_block()


            self.state = 329
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,22,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 327
                    self._errHandler.sync(self)
                    token = self._input.LA(1)
                    if token in [jacParser.T__1, jacParser.T__2, jacParser.T__4, jacParser.T__5, jacParser.T__6, jacParser.TYP_STRING, jacParser.TYP_INT, jacParser.TYP_FLOAT, jacParser.TYP_LIST, jacParser.TYP_DICT, jacParser.TYP_BOOL, jacParser.KW_TYPE, jacParser.KW_NODE, jacParser.KW_IGNORE, jacParser.KW_TAKE, jacParser.KW_SPAWN, jacParser.COLON, jacParser.DBL_COLON, jacParser.LBRACE, jacParser.KW_EDGE, jacParser.KW_ASSERT, jacParser.KW_IF, jacParser.KW_FOR, jacParser.KW_WHILE, jacParser.KW_CONTINUE, jacParser.KW_BREAK, jacParser.KW_DISENGAGE, jacParser.KW_SKIP, jacParser.KW_REPORT, jacParser.KW_DESTROY, jacParser.KW_TRY, jacParser.NOT, jacParser.PLUS, jacParser.MINUS, jacParser.STAR_MUL, jacParser.LPAREN, jacParser.LSQUARE, jacParser.FLOAT, jacParser.STRING, jacParser.BOOL, jacParser.INT, jacParser.NULL, jacParser.NAME]:
                        self.state = 325
                        self.statement()
                        pass
                    elif token in [jacParser.KW_WITH]:
                        self.state = 326
                        self.walk_activity_block()
                        pass
                    else:
                        raise NoViableAltException(self)
             
                self.state = 331
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,22,self._ctx)

            self.state = 333
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.KW_WITH:
                self.state = 332
                self.walk_exit_block()


            self.state = 335
            self.match(jacParser.RBRACE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TestContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_TEST(self):
            return self.getToken(jacParser.KW_TEST, 0)

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def KW_BY(self):
            return self.getToken(jacParser.KW_BY, 0)

        def graph_ref(self):
            return self.getTypedRuleContext(jacParser.Graph_refContext,0)


        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def graph_block(self):
            return self.getTypedRuleContext(jacParser.Graph_blockContext,0)


        def KW_WALKER(self):
            return self.getToken(jacParser.KW_WALKER, 0)

        def walker_block(self):
            return self.getTypedRuleContext(jacParser.Walker_blockContext,0)


        def walker_ref(self):
            return self.getTypedRuleContext(jacParser.Walker_refContext,0)


        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_test

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTest" ):
                listener.enterTest(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTest" ):
                listener.exitTest(self)




    def test(self):

        localctx = jacParser.TestContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_test)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 337
            self.match(jacParser.KW_TEST)
            self.state = 338
            self.match(jacParser.STRING)
            self.state = 339
            self.match(jacParser.KW_WITH)
            self.state = 343
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,24,self._ctx)
            if la_ == 1:
                self.state = 340
                self.graph_ref()
                pass

            elif la_ == 2:
                self.state = 341
                self.match(jacParser.KW_GRAPH)
                self.state = 342
                self.graph_block()
                pass


            self.state = 345
            self.match(jacParser.KW_BY)
            self.state = 356
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,27,self._ctx)
            if la_ == 1:
                self.state = 346
                self.walker_ref()
                self.state = 348
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.LPAREN:
                    self.state = 347
                    self.spawn_ctx()


                self.state = 352
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.COLON, jacParser.LBRACE]:
                    self.state = 350
                    self.code_block()
                    pass
                elif token in [jacParser.SEMI]:
                    self.state = 351
                    self.match(jacParser.SEMI)
                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 2:
                self.state = 354
                self.match(jacParser.KW_WALKER)
                self.state = 355
                self.walker_block()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NamespacesContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def name_list(self):
            return self.getTypedRuleContext(jacParser.Name_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_namespaces

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNamespaces" ):
                listener.enterNamespaces(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNamespaces" ):
                listener.exitNamespaces(self)




    def namespaces(self):

        localctx = jacParser.NamespacesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_namespaces)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 358
            self.match(jacParser.COLON)
            self.state = 359
            self.name_list()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walk_entry_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def KW_ENTRY(self):
            return self.getToken(jacParser.KW_ENTRY, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walk_entry_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalk_entry_block" ):
                listener.enterWalk_entry_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalk_entry_block" ):
                listener.exitWalk_entry_block(self)




    def walk_entry_block(self):

        localctx = jacParser.Walk_entry_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_walk_entry_block)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 361
            self.match(jacParser.KW_WITH)
            self.state = 362
            self.match(jacParser.KW_ENTRY)
            self.state = 363
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walk_exit_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def KW_EXIT(self):
            return self.getToken(jacParser.KW_EXIT, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walk_exit_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalk_exit_block" ):
                listener.enterWalk_exit_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalk_exit_block" ):
                listener.exitWalk_exit_block(self)




    def walk_exit_block(self):

        localctx = jacParser.Walk_exit_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_walk_exit_block)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 365
            self.match(jacParser.KW_WITH)
            self.state = 366
            self.match(jacParser.KW_EXIT)
            self.state = 367
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walk_activity_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def KW_ACTIVITY(self):
            return self.getToken(jacParser.KW_ACTIVITY, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walk_activity_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalk_activity_block" ):
                listener.enterWalk_activity_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalk_activity_block" ):
                listener.exitWalk_activity_block(self)




    def walk_activity_block(self):

        localctx = jacParser.Walk_activity_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_walk_activity_block)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 369
            self.match(jacParser.KW_WITH)
            self.state = 370
            self.match(jacParser.KW_ACTIVITY)
            self.state = 371
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Attr_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def attr_stmt(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Attr_stmtContext)
            else:
                return self.getTypedRuleContext(jacParser.Attr_stmtContext,i)


        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_attr_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAttr_block" ):
                listener.enterAttr_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAttr_block" ):
                listener.exitAttr_block(self)




    def attr_block(self):

        localctx = jacParser.Attr_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_attr_block)
        self._la = 0 # Token type
        try:
            self.state = 384
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.LBRACE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 373
                self.match(jacParser.LBRACE)
                self.state = 377
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==jacParser.KW_HAS or _la==jacParser.KW_CAN:
                    self.state = 374
                    self.attr_stmt()
                    self.state = 379
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 380
                self.match(jacParser.RBRACE)
                pass
            elif token in [jacParser.COLON]:
                self.enterOuterAlt(localctx, 2)
                self.state = 381
                self.match(jacParser.COLON)
                self.state = 382
                self.attr_stmt()
                pass
            elif token in [jacParser.SEMI]:
                self.enterOuterAlt(localctx, 3)
                self.state = 383
                self.match(jacParser.SEMI)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Attr_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def has_stmt(self):
            return self.getTypedRuleContext(jacParser.Has_stmtContext,0)


        def can_stmt(self):
            return self.getTypedRuleContext(jacParser.Can_stmtContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_attr_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAttr_stmt" ):
                listener.enterAttr_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAttr_stmt" ):
                listener.exitAttr_stmt(self)




    def attr_stmt(self):

        localctx = jacParser.Attr_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_attr_stmt)
        try:
            self.state = 388
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_HAS]:
                self.enterOuterAlt(localctx, 1)
                self.state = 386
                self.has_stmt()
                pass
            elif token in [jacParser.KW_CAN]:
                self.enterOuterAlt(localctx, 2)
                self.state = 387
                self.can_stmt()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Graph_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def graph_block_spawn(self):
            return self.getTypedRuleContext(jacParser.Graph_block_spawnContext,0)


        def graph_block_dot(self):
            return self.getTypedRuleContext(jacParser.Graph_block_dotContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_graph_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGraph_block" ):
                listener.enterGraph_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGraph_block" ):
                listener.exitGraph_block(self)




    def graph_block(self):

        localctx = jacParser.Graph_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_graph_block)
        try:
            self.state = 392
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,31,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 390
                self.graph_block_spawn()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 391
                self.graph_block_dot()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Graph_block_spawnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def has_root(self):
            return self.getTypedRuleContext(jacParser.Has_rootContext,0)


        def KW_SPAWN(self):
            return self.getToken(jacParser.KW_SPAWN, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_graph_block_spawn

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGraph_block_spawn" ):
                listener.enterGraph_block_spawn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGraph_block_spawn" ):
                listener.exitGraph_block_spawn(self)




    def graph_block_spawn(self):

        localctx = jacParser.Graph_block_spawnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_graph_block_spawn)
        try:
            self.state = 406
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.LBRACE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 394
                self.match(jacParser.LBRACE)
                self.state = 395
                self.has_root()
                self.state = 396
                self.match(jacParser.KW_SPAWN)
                self.state = 397
                self.code_block()
                self.state = 398
                self.match(jacParser.RBRACE)
                pass
            elif token in [jacParser.COLON]:
                self.enterOuterAlt(localctx, 2)
                self.state = 400
                self.match(jacParser.COLON)
                self.state = 401
                self.has_root()
                self.state = 402
                self.match(jacParser.KW_SPAWN)
                self.state = 403
                self.code_block()
                self.state = 404
                self.match(jacParser.SEMI)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Graph_block_dotContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def has_root(self):
            return self.getTypedRuleContext(jacParser.Has_rootContext,0)


        def dot_graph(self):
            return self.getTypedRuleContext(jacParser.Dot_graphContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_graph_block_dot

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGraph_block_dot" ):
                listener.enterGraph_block_dot(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGraph_block_dot" ):
                listener.exitGraph_block_dot(self)




    def graph_block_dot(self):

        localctx = jacParser.Graph_block_dotContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_graph_block_dot)
        try:
            self.state = 418
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.LBRACE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 408
                self.match(jacParser.LBRACE)
                self.state = 409
                self.has_root()
                self.state = 410
                self.dot_graph()
                self.state = 411
                self.match(jacParser.RBRACE)
                pass
            elif token in [jacParser.COLON]:
                self.enterOuterAlt(localctx, 2)
                self.state = 413
                self.match(jacParser.COLON)
                self.state = 414
                self.has_root()
                self.state = 415
                self.dot_graph()
                self.state = 416
                self.match(jacParser.SEMI)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Has_rootContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_HAS(self):
            return self.getToken(jacParser.KW_HAS, 0)

        def KW_ANCHOR(self):
            return self.getToken(jacParser.KW_ANCHOR, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_has_root

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHas_root" ):
                listener.enterHas_root(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHas_root" ):
                listener.exitHas_root(self)




    def has_root(self):

        localctx = jacParser.Has_rootContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_has_root)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 420
            self.match(jacParser.KW_HAS)
            self.state = 421
            self.match(jacParser.KW_ANCHOR)
            self.state = 422
            self.match(jacParser.NAME)
            self.state = 423
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Has_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_HAS(self):
            return self.getToken(jacParser.KW_HAS, 0)

        def has_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Has_assignContext)
            else:
                return self.getTypedRuleContext(jacParser.Has_assignContext,i)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def KW_PRIVATE(self):
            return self.getToken(jacParser.KW_PRIVATE, 0)

        def KW_ANCHOR(self):
            return self.getToken(jacParser.KW_ANCHOR, 0)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_has_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHas_stmt" ):
                listener.enterHas_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHas_stmt" ):
                listener.exitHas_stmt(self)




    def has_stmt(self):

        localctx = jacParser.Has_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_has_stmt)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 425
            self.match(jacParser.KW_HAS)
            self.state = 427
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.KW_PRIVATE:
                self.state = 426
                self.match(jacParser.KW_PRIVATE)


            self.state = 430
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.KW_ANCHOR:
                self.state = 429
                self.match(jacParser.KW_ANCHOR)


            self.state = 432
            self.has_assign()
            self.state = 437
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==jacParser.COMMA:
                self.state = 433
                self.match(jacParser.COMMA)
                self.state = 434
                self.has_assign()
                self.state = 439
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 440
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Has_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def EQ(self):
            return self.getToken(jacParser.EQ, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_has_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHas_assign" ):
                listener.enterHas_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHas_assign" ):
                listener.exitHas_assign(self)




    def has_assign(self):

        localctx = jacParser.Has_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_has_assign)
        try:
            self.state = 446
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,37,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 442
                self.match(jacParser.NAME)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 443
                self.match(jacParser.NAME)
                self.state = 444
                self.match(jacParser.EQ)
                self.state = 445
                self.expression()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Can_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_CAN(self):
            return self.getToken(jacParser.KW_CAN, 0)

        def dotted_name(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dotted_nameContext)
            else:
                return self.getTypedRuleContext(jacParser.Dotted_nameContext,i)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def preset_in_out(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Preset_in_outContext)
            else:
                return self.getTypedRuleContext(jacParser.Preset_in_outContext,i)


        def event_clause(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Event_clauseContext)
            else:
                return self.getTypedRuleContext(jacParser.Event_clauseContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_can_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCan_stmt" ):
                listener.enterCan_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCan_stmt" ):
                listener.exitCan_stmt(self)




    def can_stmt(self):

        localctx = jacParser.Can_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_can_stmt)
        self._la = 0 # Token type
        try:
            self.state = 475
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,42,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 448
                self.match(jacParser.KW_CAN)
                self.state = 449
                self.dotted_name()
                self.state = 453
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.DBL_COLON:
                    self.state = 450
                    self.preset_in_out()
                    self.state = 451
                    self.event_clause()


                self.state = 464
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==jacParser.COMMA:
                    self.state = 455
                    self.match(jacParser.COMMA)
                    self.state = 456
                    self.dotted_name()
                    self.state = 460
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==jacParser.DBL_COLON:
                        self.state = 457
                        self.preset_in_out()
                        self.state = 458
                        self.event_clause()


                    self.state = 466
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 467
                self.match(jacParser.SEMI)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 469
                self.match(jacParser.KW_CAN)
                self.state = 470
                self.match(jacParser.NAME)
                self.state = 472
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.KW_WITH:
                    self.state = 471
                    self.event_clause()


                self.state = 474
                self.code_block()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Event_clauseContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def KW_ENTRY(self):
            return self.getToken(jacParser.KW_ENTRY, 0)

        def KW_EXIT(self):
            return self.getToken(jacParser.KW_EXIT, 0)

        def KW_ACTIVITY(self):
            return self.getToken(jacParser.KW_ACTIVITY, 0)

        def name_list(self):
            return self.getTypedRuleContext(jacParser.Name_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_event_clause

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEvent_clause" ):
                listener.enterEvent_clause(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEvent_clause" ):
                listener.exitEvent_clause(self)




    def event_clause(self):

        localctx = jacParser.Event_clauseContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_event_clause)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 477
            self.match(jacParser.KW_WITH)
            self.state = 479
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.NAME:
                self.state = 478
                self.name_list()


            self.state = 481
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_ENTRY) | (1 << jacParser.KW_EXIT) | (1 << jacParser.KW_ACTIVITY))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Preset_in_outContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DBL_COLON(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.DBL_COLON)
            else:
                return self.getToken(jacParser.DBL_COLON, i)

        def COLON_OUT(self):
            return self.getToken(jacParser.COLON_OUT, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_preset_in_out

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPreset_in_out" ):
                listener.enterPreset_in_out(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPreset_in_out" ):
                listener.exitPreset_in_out(self)




    def preset_in_out(self):

        localctx = jacParser.Preset_in_outContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_preset_in_out)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 483
            self.match(jacParser.DBL_COLON)
            self.state = 485
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,44,self._ctx)
            if la_ == 1:
                self.state = 484
                self.expr_list()


            self.state = 490
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.DBL_COLON]:
                self.state = 487
                self.match(jacParser.DBL_COLON)
                pass
            elif token in [jacParser.COLON_OUT]:
                self.state = 488
                self.match(jacParser.COLON_OUT)
                self.state = 489
                self.expression()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dotted_nameContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.NAME)
            else:
                return self.getToken(jacParser.NAME, i)

        def DOT(self):
            return self.getToken(jacParser.DOT, 0)

        def getRuleIndex(self):
            return jacParser.RULE_dotted_name

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDotted_name" ):
                listener.enterDotted_name(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDotted_name" ):
                listener.exitDotted_name(self)




    def dotted_name(self):

        localctx = jacParser.Dotted_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_dotted_name)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 492
            self.match(jacParser.NAME)
            self.state = 493
            self.match(jacParser.DOT)
            self.state = 494
            self.match(jacParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Name_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.NAME)
            else:
                return self.getToken(jacParser.NAME, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_name_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterName_list" ):
                listener.enterName_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitName_list" ):
                listener.exitName_list(self)




    def name_list(self):

        localctx = jacParser.Name_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_name_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 496
            self.match(jacParser.NAME)
            self.state = 501
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==jacParser.COMMA:
                self.state = 497
                self.match(jacParser.COMMA)
                self.state = 498
                self.match(jacParser.NAME)
                self.state = 503
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Expr_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(jacParser.ExpressionContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_expr_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExpr_list" ):
                listener.enterExpr_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExpr_list" ):
                listener.exitExpr_list(self)




    def expr_list(self):

        localctx = jacParser.Expr_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_expr_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 504
            self.expression()
            self.state = 509
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==jacParser.COMMA:
                self.state = 505
                self.match(jacParser.COMMA)
                self.state = 506
                self.expression()
                self.state = 511
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Code_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.StatementContext)
            else:
                return self.getTypedRuleContext(jacParser.StatementContext,i)


        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def getRuleIndex(self):
            return jacParser.RULE_code_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCode_block" ):
                listener.enterCode_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCode_block" ):
                listener.exitCode_block(self)




    def code_block(self):

        localctx = jacParser.Code_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_code_block)
        self._la = 0 # Token type
        try:
            self.state = 522
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.LBRACE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 512
                self.match(jacParser.LBRACE)
                self.state = 516
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.T__1) | (1 << jacParser.T__2) | (1 << jacParser.T__4) | (1 << jacParser.T__5) | (1 << jacParser.T__6) | (1 << jacParser.TYP_STRING) | (1 << jacParser.TYP_INT) | (1 << jacParser.TYP_FLOAT) | (1 << jacParser.TYP_LIST) | (1 << jacParser.TYP_DICT) | (1 << jacParser.TYP_BOOL) | (1 << jacParser.KW_TYPE) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_IGNORE) | (1 << jacParser.KW_TAKE) | (1 << jacParser.KW_SPAWN) | (1 << jacParser.COLON) | (1 << jacParser.DBL_COLON) | (1 << jacParser.LBRACE) | (1 << jacParser.KW_EDGE) | (1 << jacParser.KW_ASSERT) | (1 << jacParser.KW_IF) | (1 << jacParser.KW_FOR) | (1 << jacParser.KW_WHILE) | (1 << jacParser.KW_CONTINUE) | (1 << jacParser.KW_BREAK))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (jacParser.KW_DISENGAGE - 64)) | (1 << (jacParser.KW_SKIP - 64)) | (1 << (jacParser.KW_REPORT - 64)) | (1 << (jacParser.KW_DESTROY - 64)) | (1 << (jacParser.KW_TRY - 64)) | (1 << (jacParser.NOT - 64)) | (1 << (jacParser.PLUS - 64)) | (1 << (jacParser.MINUS - 64)) | (1 << (jacParser.STAR_MUL - 64)) | (1 << (jacParser.LPAREN - 64)) | (1 << (jacParser.LSQUARE - 64)) | (1 << (jacParser.FLOAT - 64)) | (1 << (jacParser.STRING - 64)) | (1 << (jacParser.BOOL - 64)) | (1 << (jacParser.INT - 64)) | (1 << (jacParser.NULL - 64)) | (1 << (jacParser.NAME - 64)))) != 0):
                    self.state = 513
                    self.statement()
                    self.state = 518
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 519
                self.match(jacParser.RBRACE)
                pass
            elif token in [jacParser.COLON]:
                self.enterOuterAlt(localctx, 2)
                self.state = 520
                self.match(jacParser.COLON)
                self.state = 521
                self.statement()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Node_ctx_blockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def name_list(self):
            return self.getTypedRuleContext(jacParser.Name_listContext,0)


        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_node_ctx_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNode_ctx_block" ):
                listener.enterNode_ctx_block(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNode_ctx_block" ):
                listener.exitNode_ctx_block(self)




    def node_ctx_block(self):

        localctx = jacParser.Node_ctx_blockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_node_ctx_block)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 524
            self.name_list()
            self.state = 525
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class StatementContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def node_ctx_block(self):
            return self.getTypedRuleContext(jacParser.Node_ctx_blockContext,0)


        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def if_stmt(self):
            return self.getTypedRuleContext(jacParser.If_stmtContext,0)


        def try_stmt(self):
            return self.getTypedRuleContext(jacParser.Try_stmtContext,0)


        def for_stmt(self):
            return self.getTypedRuleContext(jacParser.For_stmtContext,0)


        def while_stmt(self):
            return self.getTypedRuleContext(jacParser.While_stmtContext,0)


        def assert_stmt(self):
            return self.getTypedRuleContext(jacParser.Assert_stmtContext,0)


        def ctrl_stmt(self):
            return self.getTypedRuleContext(jacParser.Ctrl_stmtContext,0)


        def destroy_action(self):
            return self.getTypedRuleContext(jacParser.Destroy_actionContext,0)


        def report_action(self):
            return self.getTypedRuleContext(jacParser.Report_actionContext,0)


        def walker_action(self):
            return self.getTypedRuleContext(jacParser.Walker_actionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStatement" ):
                listener.enterStatement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStatement" ):
                listener.exitStatement(self)




    def statement(self):

        localctx = jacParser.StatementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_statement)
        try:
            self.state = 545
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,50,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 527
                self.code_block()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 528
                self.node_ctx_block()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 529
                self.expression()
                self.state = 530
                self.match(jacParser.SEMI)
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 532
                self.if_stmt()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 533
                self.try_stmt()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 534
                self.for_stmt()
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 535
                self.while_stmt()
                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 536
                self.assert_stmt()
                self.state = 537
                self.match(jacParser.SEMI)
                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 539
                self.ctrl_stmt()
                self.state = 540
                self.match(jacParser.SEMI)
                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 542
                self.destroy_action()
                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 543
                self.report_action()
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 544
                self.walker_action()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class If_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_IF(self):
            return self.getToken(jacParser.KW_IF, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def elif_stmt(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Elif_stmtContext)
            else:
                return self.getTypedRuleContext(jacParser.Elif_stmtContext,i)


        def else_stmt(self):
            return self.getTypedRuleContext(jacParser.Else_stmtContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_if_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIf_stmt" ):
                listener.enterIf_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIf_stmt" ):
                listener.exitIf_stmt(self)




    def if_stmt(self):

        localctx = jacParser.If_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_if_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 547
            self.match(jacParser.KW_IF)
            self.state = 548
            self.expression()
            self.state = 549
            self.code_block()
            self.state = 553
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,51,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 550
                    self.elif_stmt() 
                self.state = 555
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,51,self._ctx)

            self.state = 557
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,52,self._ctx)
            if la_ == 1:
                self.state = 556
                self.else_stmt()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Try_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_TRY(self):
            return self.getToken(jacParser.KW_TRY, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def else_from_try(self):
            return self.getTypedRuleContext(jacParser.Else_from_tryContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_try_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTry_stmt" ):
                listener.enterTry_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTry_stmt" ):
                listener.exitTry_stmt(self)




    def try_stmt(self):

        localctx = jacParser.Try_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_try_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 559
            self.match(jacParser.KW_TRY)
            self.state = 560
            self.code_block()
            self.state = 562
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,53,self._ctx)
            if la_ == 1:
                self.state = 561
                self.else_from_try()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Else_from_tryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_ELSE(self):
            return self.getToken(jacParser.KW_ELSE, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def KW_WITH(self):
            return self.getToken(jacParser.KW_WITH, 0)

        def getRuleIndex(self):
            return jacParser.RULE_else_from_try

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterElse_from_try" ):
                listener.enterElse_from_try(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitElse_from_try" ):
                listener.exitElse_from_try(self)




    def else_from_try(self):

        localctx = jacParser.Else_from_tryContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_else_from_try)
        self._la = 0 # Token type
        try:
            self.state = 577
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,56,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 564
                self.match(jacParser.KW_ELSE)
                self.state = 568
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.LPAREN:
                    self.state = 565
                    self.match(jacParser.LPAREN)
                    self.state = 566
                    self.match(jacParser.NAME)
                    self.state = 567
                    self.match(jacParser.RPAREN)


                self.state = 570
                self.code_block()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 571
                self.match(jacParser.KW_ELSE)
                self.state = 574
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.KW_WITH:
                    self.state = 572
                    self.match(jacParser.KW_WITH)
                    self.state = 573
                    self.match(jacParser.NAME)


                self.state = 576
                self.code_block()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Elif_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_ELIF(self):
            return self.getToken(jacParser.KW_ELIF, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_elif_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterElif_stmt" ):
                listener.enterElif_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitElif_stmt" ):
                listener.exitElif_stmt(self)




    def elif_stmt(self):

        localctx = jacParser.Elif_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_elif_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 579
            self.match(jacParser.KW_ELIF)
            self.state = 580
            self.expression()
            self.state = 581
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Else_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_ELSE(self):
            return self.getToken(jacParser.KW_ELSE, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_else_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterElse_stmt" ):
                listener.enterElse_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitElse_stmt" ):
                listener.exitElse_stmt(self)




    def else_stmt(self):

        localctx = jacParser.Else_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_else_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 583
            self.match(jacParser.KW_ELSE)
            self.state = 584
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class For_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_FOR(self):
            return self.getToken(jacParser.KW_FOR, 0)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(jacParser.ExpressionContext,i)


        def KW_TO(self):
            return self.getToken(jacParser.KW_TO, 0)

        def KW_BY(self):
            return self.getToken(jacParser.KW_BY, 0)

        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def KW_IN(self):
            return self.getToken(jacParser.KW_IN, 0)

        def getRuleIndex(self):
            return jacParser.RULE_for_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFor_stmt" ):
                listener.enterFor_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFor_stmt" ):
                listener.exitFor_stmt(self)




    def for_stmt(self):

        localctx = jacParser.For_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_for_stmt)
        try:
            self.state = 600
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,57,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 586
                self.match(jacParser.KW_FOR)
                self.state = 587
                self.expression()
                self.state = 588
                self.match(jacParser.KW_TO)
                self.state = 589
                self.expression()
                self.state = 590
                self.match(jacParser.KW_BY)
                self.state = 591
                self.expression()
                self.state = 592
                self.code_block()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 594
                self.match(jacParser.KW_FOR)
                self.state = 595
                self.match(jacParser.NAME)
                self.state = 596
                self.match(jacParser.KW_IN)
                self.state = 597
                self.expression()
                self.state = 598
                self.code_block()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class While_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WHILE(self):
            return self.getToken(jacParser.KW_WHILE, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def code_block(self):
            return self.getTypedRuleContext(jacParser.Code_blockContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_while_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWhile_stmt" ):
                listener.enterWhile_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWhile_stmt" ):
                listener.exitWhile_stmt(self)




    def while_stmt(self):

        localctx = jacParser.While_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_while_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 602
            self.match(jacParser.KW_WHILE)
            self.state = 603
            self.expression()
            self.state = 604
            self.code_block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Ctrl_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_CONTINUE(self):
            return self.getToken(jacParser.KW_CONTINUE, 0)

        def KW_BREAK(self):
            return self.getToken(jacParser.KW_BREAK, 0)

        def KW_SKIP(self):
            return self.getToken(jacParser.KW_SKIP, 0)

        def getRuleIndex(self):
            return jacParser.RULE_ctrl_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCtrl_stmt" ):
                listener.enterCtrl_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCtrl_stmt" ):
                listener.exitCtrl_stmt(self)




    def ctrl_stmt(self):

        localctx = jacParser.Ctrl_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_ctrl_stmt)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 606
            _la = self._input.LA(1)
            if not(((((_la - 62)) & ~0x3f) == 0 and ((1 << (_la - 62)) & ((1 << (jacParser.KW_CONTINUE - 62)) | (1 << (jacParser.KW_BREAK - 62)) | (1 << (jacParser.KW_SKIP - 62)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Assert_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_ASSERT(self):
            return self.getToken(jacParser.KW_ASSERT, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_assert_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssert_stmt" ):
                listener.enterAssert_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssert_stmt" ):
                listener.exitAssert_stmt(self)




    def assert_stmt(self):

        localctx = jacParser.Assert_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_assert_stmt)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 608
            self.match(jacParser.KW_ASSERT)
            self.state = 609
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Destroy_actionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_DESTROY(self):
            return self.getToken(jacParser.KW_DESTROY, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_destroy_action

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDestroy_action" ):
                listener.enterDestroy_action(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDestroy_action" ):
                listener.exitDestroy_action(self)




    def destroy_action(self):

        localctx = jacParser.Destroy_actionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_destroy_action)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 611
            self.match(jacParser.KW_DESTROY)
            self.state = 612
            self.expression()
            self.state = 613
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Report_actionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_REPORT(self):
            return self.getToken(jacParser.KW_REPORT, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_report_action

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterReport_action" ):
                listener.enterReport_action(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitReport_action" ):
                listener.exitReport_action(self)




    def report_action(self):

        localctx = jacParser.Report_actionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_report_action)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 615
            self.match(jacParser.KW_REPORT)
            self.state = 616
            self.expression()
            self.state = 617
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walker_actionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ignore_action(self):
            return self.getTypedRuleContext(jacParser.Ignore_actionContext,0)


        def take_action(self):
            return self.getTypedRuleContext(jacParser.Take_actionContext,0)


        def KW_DISENGAGE(self):
            return self.getToken(jacParser.KW_DISENGAGE, 0)

        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_walker_action

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalker_action" ):
                listener.enterWalker_action(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalker_action" ):
                listener.exitWalker_action(self)




    def walker_action(self):

        localctx = jacParser.Walker_actionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_walker_action)
        try:
            self.state = 623
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_IGNORE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 619
                self.ignore_action()
                pass
            elif token in [jacParser.KW_TAKE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 620
                self.take_action()
                pass
            elif token in [jacParser.KW_DISENGAGE]:
                self.enterOuterAlt(localctx, 3)
                self.state = 621
                self.match(jacParser.KW_DISENGAGE)
                self.state = 622
                self.match(jacParser.SEMI)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Ignore_actionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_IGNORE(self):
            return self.getToken(jacParser.KW_IGNORE, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def getRuleIndex(self):
            return jacParser.RULE_ignore_action

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIgnore_action" ):
                listener.enterIgnore_action(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIgnore_action" ):
                listener.exitIgnore_action(self)




    def ignore_action(self):

        localctx = jacParser.Ignore_actionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_ignore_action)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 625
            self.match(jacParser.KW_IGNORE)
            self.state = 626
            self.expression()
            self.state = 627
            self.match(jacParser.SEMI)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Take_actionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_TAKE(self):
            return self.getToken(jacParser.KW_TAKE, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def SEMI(self):
            return self.getToken(jacParser.SEMI, 0)

        def else_stmt(self):
            return self.getTypedRuleContext(jacParser.Else_stmtContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_take_action

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTake_action" ):
                listener.enterTake_action(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTake_action" ):
                listener.exitTake_action(self)




    def take_action(self):

        localctx = jacParser.Take_actionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_take_action)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 629
            self.match(jacParser.KW_TAKE)
            self.state = 630
            self.expression()
            self.state = 633
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.SEMI]:
                self.state = 631
                self.match(jacParser.SEMI)
                pass
            elif token in [jacParser.KW_ELSE]:
                self.state = 632
                self.else_stmt()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def connect(self):
            return self.getTypedRuleContext(jacParser.ConnectContext,0)


        def assignment(self):
            return self.getTypedRuleContext(jacParser.AssignmentContext,0)


        def copy_assign(self):
            return self.getTypedRuleContext(jacParser.Copy_assignContext,0)


        def inc_assign(self):
            return self.getTypedRuleContext(jacParser.Inc_assignContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_expression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExpression" ):
                listener.enterExpression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExpression" ):
                listener.exitExpression(self)




    def expression(self):

        localctx = jacParser.ExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_expression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 635
            self.connect()
            self.state = 639
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,60,self._ctx)
            if la_ == 1:
                self.state = 636
                self.assignment()

            elif la_ == 2:
                self.state = 637
                self.copy_assign()

            elif la_ == 3:
                self.state = 638
                self.inc_assign()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignmentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EQ(self):
            return self.getToken(jacParser.EQ, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_assignment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssignment" ):
                listener.enterAssignment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssignment" ):
                listener.exitAssignment(self)




    def assignment(self):

        localctx = jacParser.AssignmentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_assignment)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 641
            self.match(jacParser.EQ)
            self.state = 642
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Copy_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def CPY_EQ(self):
            return self.getToken(jacParser.CPY_EQ, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_copy_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCopy_assign" ):
                listener.enterCopy_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCopy_assign" ):
                listener.exitCopy_assign(self)




    def copy_assign(self):

        localctx = jacParser.Copy_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_copy_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 644
            self.match(jacParser.CPY_EQ)
            self.state = 645
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Inc_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def PEQ(self):
            return self.getToken(jacParser.PEQ, 0)

        def MEQ(self):
            return self.getToken(jacParser.MEQ, 0)

        def TEQ(self):
            return self.getToken(jacParser.TEQ, 0)

        def DEQ(self):
            return self.getToken(jacParser.DEQ, 0)

        def getRuleIndex(self):
            return jacParser.RULE_inc_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInc_assign" ):
                listener.enterInc_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInc_assign" ):
                listener.exitInc_assign(self)




    def inc_assign(self):

        localctx = jacParser.Inc_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_inc_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 647
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.PEQ) | (1 << jacParser.MEQ) | (1 << jacParser.TEQ) | (1 << jacParser.DEQ))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 648
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ConnectContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def logical(self):
            return self.getTypedRuleContext(jacParser.LogicalContext,0)


        def edge_ref(self):
            return self.getTypedRuleContext(jacParser.Edge_refContext,0)


        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def NOT(self):
            return self.getToken(jacParser.NOT, 0)

        def getRuleIndex(self):
            return jacParser.RULE_connect

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConnect" ):
                listener.enterConnect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConnect" ):
                listener.exitConnect(self)




    def connect(self):

        localctx = jacParser.ConnectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 98, self.RULE_connect)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 650
            self.logical()
            self.state = 657
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,62,self._ctx)
            if la_ == 1:
                self.state = 652
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.NOT:
                    self.state = 651
                    self.match(jacParser.NOT)


                self.state = 654
                self.edge_ref()
                self.state = 655
                self.expression()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LogicalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def compare(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.CompareContext)
            else:
                return self.getTypedRuleContext(jacParser.CompareContext,i)


        def KW_AND(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.KW_AND)
            else:
                return self.getToken(jacParser.KW_AND, i)

        def KW_OR(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.KW_OR)
            else:
                return self.getToken(jacParser.KW_OR, i)

        def getRuleIndex(self):
            return jacParser.RULE_logical

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLogical" ):
                listener.enterLogical(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLogical" ):
                listener.exitLogical(self)




    def logical(self):

        localctx = jacParser.LogicalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_logical)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 659
            self.compare()
            self.state = 664
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,63,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 660
                    _la = self._input.LA(1)
                    if not(_la==jacParser.KW_AND or _la==jacParser.KW_OR):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 661
                    self.compare() 
                self.state = 666
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,63,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CompareContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NOT(self):
            return self.getToken(jacParser.NOT, 0)

        def compare(self):
            return self.getTypedRuleContext(jacParser.CompareContext,0)


        def arithmetic(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.ArithmeticContext)
            else:
                return self.getTypedRuleContext(jacParser.ArithmeticContext,i)


        def cmp_op(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Cmp_opContext)
            else:
                return self.getTypedRuleContext(jacParser.Cmp_opContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_compare

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCompare" ):
                listener.enterCompare(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCompare" ):
                listener.exitCompare(self)




    def compare(self):

        localctx = jacParser.CompareContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_compare)
        try:
            self.state = 678
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.NOT]:
                self.enterOuterAlt(localctx, 1)
                self.state = 667
                self.match(jacParser.NOT)
                self.state = 668
                self.compare()
                pass
            elif token in [jacParser.T__1, jacParser.T__2, jacParser.T__4, jacParser.T__5, jacParser.T__6, jacParser.TYP_STRING, jacParser.TYP_INT, jacParser.TYP_FLOAT, jacParser.TYP_LIST, jacParser.TYP_DICT, jacParser.TYP_BOOL, jacParser.KW_TYPE, jacParser.KW_NODE, jacParser.KW_SPAWN, jacParser.DBL_COLON, jacParser.LBRACE, jacParser.KW_EDGE, jacParser.PLUS, jacParser.MINUS, jacParser.STAR_MUL, jacParser.LPAREN, jacParser.LSQUARE, jacParser.FLOAT, jacParser.STRING, jacParser.BOOL, jacParser.INT, jacParser.NULL, jacParser.NAME]:
                self.enterOuterAlt(localctx, 2)
                self.state = 669
                self.arithmetic()
                self.state = 675
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,64,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 670
                        self.cmp_op()
                        self.state = 671
                        self.arithmetic() 
                    self.state = 677
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,64,self._ctx)

                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Cmp_opContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EE(self):
            return self.getToken(jacParser.EE, 0)

        def LT(self):
            return self.getToken(jacParser.LT, 0)

        def GT(self):
            return self.getToken(jacParser.GT, 0)

        def LTE(self):
            return self.getToken(jacParser.LTE, 0)

        def GTE(self):
            return self.getToken(jacParser.GTE, 0)

        def NE(self):
            return self.getToken(jacParser.NE, 0)

        def KW_IN(self):
            return self.getToken(jacParser.KW_IN, 0)

        def nin(self):
            return self.getTypedRuleContext(jacParser.NinContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_cmp_op

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCmp_op" ):
                listener.enterCmp_op(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCmp_op" ):
                listener.exitCmp_op(self)




    def cmp_op(self):

        localctx = jacParser.Cmp_opContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_cmp_op)
        try:
            self.state = 688
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.EE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 680
                self.match(jacParser.EE)
                pass
            elif token in [jacParser.LT]:
                self.enterOuterAlt(localctx, 2)
                self.state = 681
                self.match(jacParser.LT)
                pass
            elif token in [jacParser.GT]:
                self.enterOuterAlt(localctx, 3)
                self.state = 682
                self.match(jacParser.GT)
                pass
            elif token in [jacParser.LTE]:
                self.enterOuterAlt(localctx, 4)
                self.state = 683
                self.match(jacParser.LTE)
                pass
            elif token in [jacParser.GTE]:
                self.enterOuterAlt(localctx, 5)
                self.state = 684
                self.match(jacParser.GTE)
                pass
            elif token in [jacParser.NE]:
                self.enterOuterAlt(localctx, 6)
                self.state = 685
                self.match(jacParser.NE)
                pass
            elif token in [jacParser.KW_IN]:
                self.enterOuterAlt(localctx, 7)
                self.state = 686
                self.match(jacParser.KW_IN)
                pass
            elif token in [jacParser.NOT]:
                self.enterOuterAlt(localctx, 8)
                self.state = 687
                self.nin()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NinContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NOT(self):
            return self.getToken(jacParser.NOT, 0)

        def KW_IN(self):
            return self.getToken(jacParser.KW_IN, 0)

        def getRuleIndex(self):
            return jacParser.RULE_nin

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNin" ):
                listener.enterNin(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNin" ):
                listener.exitNin(self)




    def nin(self):

        localctx = jacParser.NinContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_nin)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 690
            self.match(jacParser.NOT)
            self.state = 691
            self.match(jacParser.KW_IN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArithmeticContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.TermContext)
            else:
                return self.getTypedRuleContext(jacParser.TermContext,i)


        def PLUS(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.PLUS)
            else:
                return self.getToken(jacParser.PLUS, i)

        def MINUS(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.MINUS)
            else:
                return self.getToken(jacParser.MINUS, i)

        def getRuleIndex(self):
            return jacParser.RULE_arithmetic

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArithmetic" ):
                listener.enterArithmetic(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArithmetic" ):
                listener.exitArithmetic(self)




    def arithmetic(self):

        localctx = jacParser.ArithmeticContext(self, self._ctx, self.state)
        self.enterRule(localctx, 108, self.RULE_arithmetic)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 693
            self.term()
            self.state = 698
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,67,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 694
                    _la = self._input.LA(1)
                    if not(_la==jacParser.PLUS or _la==jacParser.MINUS):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 695
                    self.term() 
                self.state = 700
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,67,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TermContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def factor(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.FactorContext)
            else:
                return self.getTypedRuleContext(jacParser.FactorContext,i)


        def STAR_MUL(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.STAR_MUL)
            else:
                return self.getToken(jacParser.STAR_MUL, i)

        def DIV(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.DIV)
            else:
                return self.getToken(jacParser.DIV, i)

        def MOD(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.MOD)
            else:
                return self.getToken(jacParser.MOD, i)

        def getRuleIndex(self):
            return jacParser.RULE_term

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTerm" ):
                listener.enterTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTerm" ):
                listener.exitTerm(self)




    def term(self):

        localctx = jacParser.TermContext(self, self._ctx, self.state)
        self.enterRule(localctx, 110, self.RULE_term)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 701
            self.factor()
            self.state = 706
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,68,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 702
                    _la = self._input.LA(1)
                    if not(((((_la - 85)) & ~0x3f) == 0 and ((1 << (_la - 85)) & ((1 << (jacParser.STAR_MUL - 85)) | (1 << (jacParser.DIV - 85)) | (1 << (jacParser.MOD - 85)))) != 0)):
                        self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 703
                    self.factor() 
                self.state = 708
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,68,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FactorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def factor(self):
            return self.getTypedRuleContext(jacParser.FactorContext,0)


        def PLUS(self):
            return self.getToken(jacParser.PLUS, 0)

        def MINUS(self):
            return self.getToken(jacParser.MINUS, 0)

        def power(self):
            return self.getTypedRuleContext(jacParser.PowerContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_factor

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFactor" ):
                listener.enterFactor(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFactor" ):
                listener.exitFactor(self)




    def factor(self):

        localctx = jacParser.FactorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 112, self.RULE_factor)
        self._la = 0 # Token type
        try:
            self.state = 712
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,69,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 709
                _la = self._input.LA(1)
                if not(_la==jacParser.PLUS or _la==jacParser.MINUS):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 710
                self.factor()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 711
                self.power()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class PowerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def atom(self):
            return self.getTypedRuleContext(jacParser.AtomContext,0)


        def POW(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.POW)
            else:
                return self.getToken(jacParser.POW, i)

        def factor(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.FactorContext)
            else:
                return self.getTypedRuleContext(jacParser.FactorContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_power

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPower" ):
                listener.enterPower(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPower" ):
                listener.exitPower(self)




    def power(self):

        localctx = jacParser.PowerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 114, self.RULE_power)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 714
            self.atom(0)
            self.state = 719
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,70,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 715
                    self.match(jacParser.POW)
                    self.state = 716
                    self.factor() 
                self.state = 721
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,70,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtomContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def INT(self):
            return self.getToken(jacParser.INT, 0)

        def FLOAT(self):
            return self.getToken(jacParser.FLOAT, 0)

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def BOOL(self):
            return self.getToken(jacParser.BOOL, 0)

        def NULL(self):
            return self.getToken(jacParser.NULL, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def node_edge_ref(self):
            return self.getTypedRuleContext(jacParser.Node_edge_refContext,0)


        def list_val(self):
            return self.getTypedRuleContext(jacParser.List_valContext,0)


        def dict_val(self):
            return self.getTypedRuleContext(jacParser.Dict_valContext,0)


        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def spawn(self):
            return self.getTypedRuleContext(jacParser.SpawnContext,0)


        def ref(self):
            return self.getTypedRuleContext(jacParser.RefContext,0)


        def deref(self):
            return self.getTypedRuleContext(jacParser.DerefContext,0)


        def any_type(self):
            return self.getTypedRuleContext(jacParser.Any_typeContext,0)


        def atom(self):
            return self.getTypedRuleContext(jacParser.AtomContext,0)


        def atom_trailer(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Atom_trailerContext)
            else:
                return self.getTypedRuleContext(jacParser.Atom_trailerContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_atom

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtom" ):
                listener.enterAtom(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtom" ):
                listener.exitAtom(self)



    def atom(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = jacParser.AtomContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 116
        self.enterRecursionRule(localctx, 116, self.RULE_atom, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 745
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,72,self._ctx)
            if la_ == 1:
                self.state = 723
                self.match(jacParser.INT)
                pass

            elif la_ == 2:
                self.state = 724
                self.match(jacParser.FLOAT)
                pass

            elif la_ == 3:
                self.state = 725
                self.match(jacParser.STRING)
                pass

            elif la_ == 4:
                self.state = 726
                self.match(jacParser.BOOL)
                pass

            elif la_ == 5:
                self.state = 727
                self.match(jacParser.NULL)
                pass

            elif la_ == 6:
                self.state = 728
                self.match(jacParser.NAME)
                pass

            elif la_ == 7:
                self.state = 729
                self.node_edge_ref()
                pass

            elif la_ == 8:
                self.state = 730
                self.list_val()
                pass

            elif la_ == 9:
                self.state = 731
                self.dict_val()
                pass

            elif la_ == 10:
                self.state = 732
                self.match(jacParser.LPAREN)
                self.state = 733
                self.expression()
                self.state = 734
                self.match(jacParser.RPAREN)
                pass

            elif la_ == 11:
                self.state = 736
                self.match(jacParser.DBL_COLON)
                self.state = 737
                self.match(jacParser.NAME)
                self.state = 739
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,71,self._ctx)
                if la_ == 1:
                    self.state = 738
                    self.spawn_ctx()


                pass

            elif la_ == 12:
                self.state = 741
                self.spawn()
                pass

            elif la_ == 13:
                self.state = 742
                self.ref()
                pass

            elif la_ == 14:
                self.state = 743
                self.deref()
                pass

            elif la_ == 15:
                self.state = 744
                self.any_type()
                pass


            self._ctx.stop = self._input.LT(-1)
            self.state = 755
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,74,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = jacParser.AtomContext(self, _parentctx, _parentState)
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_atom)
                    self.state = 747
                    if not self.precpred(self._ctx, 5):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 5)")
                    self.state = 749 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 748
                            self.atom_trailer()

                        else:
                            raise NoViableAltException(self)
                        self.state = 751 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,73,self._ctx)
             
                self.state = 757
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,74,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class Atom_trailerContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def DOT(self):
            return self.getToken(jacParser.DOT, 0)

        def built_in(self):
            return self.getTypedRuleContext(jacParser.Built_inContext,0)


        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def index_slice(self):
            return self.getTypedRuleContext(jacParser.Index_sliceContext,0)


        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_atom_trailer

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtom_trailer" ):
                listener.enterAtom_trailer(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtom_trailer" ):
                listener.exitAtom_trailer(self)




    def atom_trailer(self):

        localctx = jacParser.Atom_trailerContext(self, self._ctx, self.state)
        self.enterRule(localctx, 118, self.RULE_atom_trailer)
        self._la = 0 # Token type
        try:
            self.state = 773
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,77,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 758
                self.match(jacParser.DOT)
                self.state = 759
                self.built_in()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 760
                self.match(jacParser.DOT)
                self.state = 761
                self.match(jacParser.NAME)
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 762
                self.index_slice()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 763
                self.match(jacParser.LPAREN)
                self.state = 765
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.T__1) | (1 << jacParser.T__2) | (1 << jacParser.T__4) | (1 << jacParser.T__5) | (1 << jacParser.T__6) | (1 << jacParser.TYP_STRING) | (1 << jacParser.TYP_INT) | (1 << jacParser.TYP_FLOAT) | (1 << jacParser.TYP_LIST) | (1 << jacParser.TYP_DICT) | (1 << jacParser.TYP_BOOL) | (1 << jacParser.KW_TYPE) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_SPAWN) | (1 << jacParser.DBL_COLON) | (1 << jacParser.LBRACE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 70)) & ~0x3f) == 0 and ((1 << (_la - 70)) & ((1 << (jacParser.NOT - 70)) | (1 << (jacParser.PLUS - 70)) | (1 << (jacParser.MINUS - 70)) | (1 << (jacParser.STAR_MUL - 70)) | (1 << (jacParser.LPAREN - 70)) | (1 << (jacParser.LSQUARE - 70)) | (1 << (jacParser.FLOAT - 70)) | (1 << (jacParser.STRING - 70)) | (1 << (jacParser.BOOL - 70)) | (1 << (jacParser.INT - 70)) | (1 << (jacParser.NULL - 70)) | (1 << (jacParser.NAME - 70)))) != 0):
                    self.state = 764
                    self.expr_list()


                self.state = 767
                self.match(jacParser.RPAREN)
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 768
                self.match(jacParser.DBL_COLON)
                self.state = 769
                self.match(jacParser.NAME)
                self.state = 771
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,76,self._ctx)
                if la_ == 1:
                    self.state = 770
                    self.spawn_ctx()


                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class RefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRef" ):
                listener.enterRef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRef" ):
                listener.exitRef(self)




    def ref(self):

        localctx = jacParser.RefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 120, self.RULE_ref)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 775
            self.match(jacParser.T__1)
            self.state = 776
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DerefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def STAR_MUL(self):
            return self.getToken(jacParser.STAR_MUL, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_deref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDeref" ):
                listener.enterDeref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDeref" ):
                listener.exitDeref(self)




    def deref(self):

        localctx = jacParser.DerefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 122, self.RULE_deref)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 778
            self.match(jacParser.STAR_MUL)
            self.state = 779
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def string_built_in(self):
            return self.getTypedRuleContext(jacParser.String_built_inContext,0)


        def dict_built_in(self):
            return self.getTypedRuleContext(jacParser.Dict_built_inContext,0)


        def list_built_in(self):
            return self.getTypedRuleContext(jacParser.List_built_inContext,0)


        def obj_built_in(self):
            return self.getTypedRuleContext(jacParser.Obj_built_inContext,0)


        def cast_built_in(self):
            return self.getTypedRuleContext(jacParser.Cast_built_inContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBuilt_in" ):
                listener.enterBuilt_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBuilt_in" ):
                listener.exitBuilt_in(self)




    def built_in(self):

        localctx = jacParser.Built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 124, self.RULE_built_in)
        try:
            self.state = 787
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,78,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)

                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 782
                self.string_built_in()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 783
                self.dict_built_in()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 784
                self.list_built_in()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 785
                self.obj_built_in()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 786
                self.cast_built_in()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Cast_built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def any_type(self):
            return self.getTypedRuleContext(jacParser.Any_typeContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_cast_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCast_built_in" ):
                listener.enterCast_built_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCast_built_in" ):
                listener.exitCast_built_in(self)




    def cast_built_in(self):

        localctx = jacParser.Cast_built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 126, self.RULE_cast_built_in)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 789
            self.any_type()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Obj_built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_CONTEXT(self):
            return self.getToken(jacParser.KW_CONTEXT, 0)

        def KW_INFO(self):
            return self.getToken(jacParser.KW_INFO, 0)

        def KW_DETAILS(self):
            return self.getToken(jacParser.KW_DETAILS, 0)

        def getRuleIndex(self):
            return jacParser.RULE_obj_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterObj_built_in" ):
                listener.enterObj_built_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitObj_built_in" ):
                listener.exitObj_built_in(self)




    def obj_built_in(self):

        localctx = jacParser.Obj_built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 128, self.RULE_obj_built_in)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 791
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_CONTEXT) | (1 << jacParser.KW_INFO) | (1 << jacParser.KW_DETAILS))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dict_built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_KEYS(self):
            return self.getToken(jacParser.KW_KEYS, 0)

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def name_list(self):
            return self.getTypedRuleContext(jacParser.Name_listContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def TYP_DICT(self):
            return self.getToken(jacParser.TYP_DICT, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def DICT_DBL_COLON(self):
            return self.getToken(jacParser.DICT_DBL_COLON, 0)

        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def getRuleIndex(self):
            return jacParser.RULE_dict_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDict_built_in" ):
                listener.enterDict_built_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDict_built_in" ):
                listener.exitDict_built_in(self)




    def dict_built_in(self):

        localctx = jacParser.Dict_built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 130, self.RULE_dict_built_in)
        self._la = 0 # Token type
        try:
            self.state = 810
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_KEYS]:
                self.enterOuterAlt(localctx, 1)
                self.state = 793
                self.match(jacParser.KW_KEYS)
                pass
            elif token in [jacParser.LBRACE]:
                self.enterOuterAlt(localctx, 2)
                self.state = 794
                self.match(jacParser.LBRACE)
                self.state = 795
                self.name_list()
                self.state = 796
                self.match(jacParser.RBRACE)
                pass
            elif token in [jacParser.TYP_DICT, jacParser.DICT_DBL_COLON]:
                self.enterOuterAlt(localctx, 3)
                self.state = 801
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.TYP_DICT]:
                    self.state = 798
                    self.match(jacParser.TYP_DICT)
                    self.state = 799
                    self.match(jacParser.DBL_COLON)
                    pass
                elif token in [jacParser.DICT_DBL_COLON]:
                    self.state = 800
                    self.match(jacParser.DICT_DBL_COLON)
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 803
                _la = self._input.LA(1)
                if not(_la==jacParser.KW_KEYS or _la==jacParser.NAME):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 808
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,80,self._ctx)
                if la_ == 1:
                    self.state = 804
                    self.match(jacParser.LPAREN)
                    self.state = 805
                    self.expr_list()
                    self.state = 806
                    self.match(jacParser.RPAREN)


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class List_built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_LENGTH(self):
            return self.getToken(jacParser.KW_LENGTH, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def TYP_LIST(self):
            return self.getToken(jacParser.TYP_LIST, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def LIST_DBL_COLON(self):
            return self.getToken(jacParser.LIST_DBL_COLON, 0)

        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def getRuleIndex(self):
            return jacParser.RULE_list_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterList_built_in" ):
                listener.enterList_built_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitList_built_in" ):
                listener.exitList_built_in(self)




    def list_built_in(self):

        localctx = jacParser.List_built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 132, self.RULE_list_built_in)
        try:
            self.state = 825
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_LENGTH]:
                self.enterOuterAlt(localctx, 1)
                self.state = 812
                self.match(jacParser.KW_LENGTH)
                pass
            elif token in [jacParser.TYP_LIST, jacParser.LIST_DBL_COLON]:
                self.enterOuterAlt(localctx, 2)
                self.state = 816
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.TYP_LIST]:
                    self.state = 813
                    self.match(jacParser.TYP_LIST)
                    self.state = 814
                    self.match(jacParser.DBL_COLON)
                    pass
                elif token in [jacParser.LIST_DBL_COLON]:
                    self.state = 815
                    self.match(jacParser.LIST_DBL_COLON)
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 818
                self.match(jacParser.NAME)
                self.state = 823
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,83,self._ctx)
                if la_ == 1:
                    self.state = 819
                    self.match(jacParser.LPAREN)
                    self.state = 820
                    self.expr_list()
                    self.state = 821
                    self.match(jacParser.RPAREN)


                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class String_built_inContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def TYP_STRING(self):
            return self.getToken(jacParser.TYP_STRING, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def STR_DBL_COLON(self):
            return self.getToken(jacParser.STR_DBL_COLON, 0)

        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def getRuleIndex(self):
            return jacParser.RULE_string_built_in

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterString_built_in" ):
                listener.enterString_built_in(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitString_built_in" ):
                listener.exitString_built_in(self)




    def string_built_in(self):

        localctx = jacParser.String_built_inContext(self, self._ctx, self.state)
        self.enterRule(localctx, 134, self.RULE_string_built_in)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 830
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.TYP_STRING]:
                self.state = 827
                self.match(jacParser.TYP_STRING)
                self.state = 828
                self.match(jacParser.DBL_COLON)
                pass
            elif token in [jacParser.STR_DBL_COLON]:
                self.state = 829
                self.match(jacParser.STR_DBL_COLON)
                pass
            else:
                raise NoViableAltException(self)

            self.state = 832
            self.match(jacParser.NAME)
            self.state = 837
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,86,self._ctx)
            if la_ == 1:
                self.state = 833
                self.match(jacParser.LPAREN)
                self.state = 834
                self.expr_list()
                self.state = 835
                self.match(jacParser.RPAREN)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Node_edge_refContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def node_ref(self):
            return self.getTypedRuleContext(jacParser.Node_refContext,0)


        def filter_ctx(self):
            return self.getTypedRuleContext(jacParser.Filter_ctxContext,0)


        def edge_ref(self):
            return self.getTypedRuleContext(jacParser.Edge_refContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_node_edge_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNode_edge_ref" ):
                listener.enterNode_edge_ref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNode_edge_ref" ):
                listener.exitNode_edge_ref(self)




    def node_edge_ref(self):

        localctx = jacParser.Node_edge_refContext(self, self._ctx, self.state)
        self.enterRule(localctx, 136, self.RULE_node_edge_ref)
        try:
            self.state = 850
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_NODE]:
                self.enterOuterAlt(localctx, 1)
                self.state = 839
                self.node_ref()
                self.state = 841
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,87,self._ctx)
                if la_ == 1:
                    self.state = 840
                    self.filter_ctx()


                pass
            elif token in [jacParser.T__2, jacParser.T__4, jacParser.T__5, jacParser.T__6, jacParser.MINUS]:
                self.enterOuterAlt(localctx, 2)
                self.state = 843
                self.edge_ref()
                self.state = 848
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,89,self._ctx)
                if la_ == 1:
                    self.state = 844
                    self.node_ref()
                    self.state = 846
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,88,self._ctx)
                    if la_ == 1:
                        self.state = 845
                        self.filter_ctx()




                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Node_refContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def getRuleIndex(self):
            return jacParser.RULE_node_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNode_ref" ):
                listener.enterNode_ref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNode_ref" ):
                listener.exitNode_ref(self)




    def node_ref(self):

        localctx = jacParser.Node_refContext(self, self._ctx, self.state)
        self.enterRule(localctx, 138, self.RULE_node_ref)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 852
            self.match(jacParser.KW_NODE)
            self.state = 853
            self.match(jacParser.DBL_COLON)
            self.state = 854
            self.match(jacParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walker_refContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_WALKER(self):
            return self.getToken(jacParser.KW_WALKER, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def getRuleIndex(self):
            return jacParser.RULE_walker_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalker_ref" ):
                listener.enterWalker_ref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalker_ref" ):
                listener.exitWalker_ref(self)




    def walker_ref(self):

        localctx = jacParser.Walker_refContext(self, self._ctx, self.state)
        self.enterRule(localctx, 140, self.RULE_walker_ref)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 856
            self.match(jacParser.KW_WALKER)
            self.state = 857
            self.match(jacParser.DBL_COLON)
            self.state = 858
            self.match(jacParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Graph_refContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def DBL_COLON(self):
            return self.getToken(jacParser.DBL_COLON, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def getRuleIndex(self):
            return jacParser.RULE_graph_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGraph_ref" ):
                listener.enterGraph_ref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGraph_ref" ):
                listener.exitGraph_ref(self)




    def graph_ref(self):

        localctx = jacParser.Graph_refContext(self, self._ctx, self.state)
        self.enterRule(localctx, 142, self.RULE_graph_ref)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 860
            self.match(jacParser.KW_GRAPH)
            self.state = 861
            self.match(jacParser.DBL_COLON)
            self.state = 862
            self.match(jacParser.NAME)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Edge_refContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def edge_to(self):
            return self.getTypedRuleContext(jacParser.Edge_toContext,0)


        def edge_from(self):
            return self.getTypedRuleContext(jacParser.Edge_fromContext,0)


        def edge_any(self):
            return self.getTypedRuleContext(jacParser.Edge_anyContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_edge_ref

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEdge_ref" ):
                listener.enterEdge_ref(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEdge_ref" ):
                listener.exitEdge_ref(self)




    def edge_ref(self):

        localctx = jacParser.Edge_refContext(self, self._ctx, self.state)
        self.enterRule(localctx, 144, self.RULE_edge_ref)
        try:
            self.state = 867
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,91,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 864
                self.edge_to()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 865
                self.edge_from()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 866
                self.edge_any()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Edge_toContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MINUS(self):
            return self.getToken(jacParser.MINUS, 0)

        def LSQUARE(self):
            return self.getToken(jacParser.LSQUARE, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def RSQUARE(self):
            return self.getToken(jacParser.RSQUARE, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def filter_ctx(self):
            return self.getTypedRuleContext(jacParser.Filter_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_edge_to

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEdge_to" ):
                listener.enterEdge_to(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEdge_to" ):
                listener.exitEdge_to(self)




    def edge_to(self):

        localctx = jacParser.Edge_toContext(self, self._ctx, self.state)
        self.enterRule(localctx, 146, self.RULE_edge_to)
        self._la = 0 # Token type
        try:
            self.state = 881
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.T__2]:
                self.enterOuterAlt(localctx, 1)
                self.state = 869
                self.match(jacParser.T__2)
                pass
            elif token in [jacParser.MINUS]:
                self.enterOuterAlt(localctx, 2)
                self.state = 870
                self.match(jacParser.MINUS)
                self.state = 878
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.LSQUARE:
                    self.state = 871
                    self.match(jacParser.LSQUARE)
                    self.state = 872
                    self.match(jacParser.NAME)
                    self.state = 875
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,92,self._ctx)
                    if la_ == 1:
                        self.state = 873
                        self.spawn_ctx()

                    elif la_ == 2:
                        self.state = 874
                        self.filter_ctx()


                    self.state = 877
                    self.match(jacParser.RSQUARE)


                self.state = 880
                self.match(jacParser.T__3)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Edge_fromContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def MINUS(self):
            return self.getToken(jacParser.MINUS, 0)

        def LSQUARE(self):
            return self.getToken(jacParser.LSQUARE, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def RSQUARE(self):
            return self.getToken(jacParser.RSQUARE, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def filter_ctx(self):
            return self.getTypedRuleContext(jacParser.Filter_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_edge_from

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEdge_from" ):
                listener.enterEdge_from(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEdge_from" ):
                listener.exitEdge_from(self)




    def edge_from(self):

        localctx = jacParser.Edge_fromContext(self, self._ctx, self.state)
        self.enterRule(localctx, 148, self.RULE_edge_from)
        self._la = 0 # Token type
        try:
            self.state = 895
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.T__4]:
                self.enterOuterAlt(localctx, 1)
                self.state = 883
                self.match(jacParser.T__4)
                pass
            elif token in [jacParser.T__5]:
                self.enterOuterAlt(localctx, 2)
                self.state = 884
                self.match(jacParser.T__5)
                self.state = 892
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.LSQUARE:
                    self.state = 885
                    self.match(jacParser.LSQUARE)
                    self.state = 886
                    self.match(jacParser.NAME)
                    self.state = 889
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,95,self._ctx)
                    if la_ == 1:
                        self.state = 887
                        self.spawn_ctx()

                    elif la_ == 2:
                        self.state = 888
                        self.filter_ctx()


                    self.state = 891
                    self.match(jacParser.RSQUARE)


                self.state = 894
                self.match(jacParser.MINUS)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Edge_anyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(jacParser.LSQUARE, 0)

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def RSQUARE(self):
            return self.getToken(jacParser.RSQUARE, 0)

        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def filter_ctx(self):
            return self.getTypedRuleContext(jacParser.Filter_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_edge_any

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEdge_any" ):
                listener.enterEdge_any(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEdge_any" ):
                listener.exitEdge_any(self)




    def edge_any(self):

        localctx = jacParser.Edge_anyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 150, self.RULE_edge_any)
        self._la = 0 # Token type
        try:
            self.state = 909
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.T__6]:
                self.enterOuterAlt(localctx, 1)
                self.state = 897
                self.match(jacParser.T__6)
                pass
            elif token in [jacParser.T__5]:
                self.enterOuterAlt(localctx, 2)
                self.state = 898
                self.match(jacParser.T__5)
                self.state = 906
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.LSQUARE:
                    self.state = 899
                    self.match(jacParser.LSQUARE)
                    self.state = 900
                    self.match(jacParser.NAME)
                    self.state = 903
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,98,self._ctx)
                    if la_ == 1:
                        self.state = 901
                        self.spawn_ctx()

                    elif la_ == 2:
                        self.state = 902
                        self.filter_ctx()


                    self.state = 905
                    self.match(jacParser.RSQUARE)


                self.state = 908
                self.match(jacParser.T__3)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class List_valContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(jacParser.LSQUARE, 0)

        def RSQUARE(self):
            return self.getToken(jacParser.RSQUARE, 0)

        def expr_list(self):
            return self.getTypedRuleContext(jacParser.Expr_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_list_val

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterList_val" ):
                listener.enterList_val(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitList_val" ):
                listener.exitList_val(self)




    def list_val(self):

        localctx = jacParser.List_valContext(self, self._ctx, self.state)
        self.enterRule(localctx, 152, self.RULE_list_val)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 911
            self.match(jacParser.LSQUARE)
            self.state = 913
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.T__1) | (1 << jacParser.T__2) | (1 << jacParser.T__4) | (1 << jacParser.T__5) | (1 << jacParser.T__6) | (1 << jacParser.TYP_STRING) | (1 << jacParser.TYP_INT) | (1 << jacParser.TYP_FLOAT) | (1 << jacParser.TYP_LIST) | (1 << jacParser.TYP_DICT) | (1 << jacParser.TYP_BOOL) | (1 << jacParser.KW_TYPE) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_SPAWN) | (1 << jacParser.DBL_COLON) | (1 << jacParser.LBRACE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 70)) & ~0x3f) == 0 and ((1 << (_la - 70)) & ((1 << (jacParser.NOT - 70)) | (1 << (jacParser.PLUS - 70)) | (1 << (jacParser.MINUS - 70)) | (1 << (jacParser.STAR_MUL - 70)) | (1 << (jacParser.LPAREN - 70)) | (1 << (jacParser.LSQUARE - 70)) | (1 << (jacParser.FLOAT - 70)) | (1 << (jacParser.STRING - 70)) | (1 << (jacParser.BOOL - 70)) | (1 << (jacParser.INT - 70)) | (1 << (jacParser.NULL - 70)) | (1 << (jacParser.NAME - 70)))) != 0):
                self.state = 912
                self.expr_list()


            self.state = 915
            self.match(jacParser.RSQUARE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Index_sliceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self):
            return self.getToken(jacParser.LSQUARE, 0)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(jacParser.ExpressionContext,i)


        def RSQUARE(self):
            return self.getToken(jacParser.RSQUARE, 0)

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def getRuleIndex(self):
            return jacParser.RULE_index_slice

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIndex_slice" ):
                listener.enterIndex_slice(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIndex_slice" ):
                listener.exitIndex_slice(self)




    def index_slice(self):

        localctx = jacParser.Index_sliceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 154, self.RULE_index_slice)
        try:
            self.state = 927
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,102,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 917
                self.match(jacParser.LSQUARE)
                self.state = 918
                self.expression()
                self.state = 919
                self.match(jacParser.RSQUARE)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 921
                self.match(jacParser.LSQUARE)
                self.state = 922
                self.expression()
                self.state = 923
                self.match(jacParser.COLON)
                self.state = 924
                self.expression()
                self.state = 925
                self.match(jacParser.RSQUARE)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dict_valContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def kv_pair(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Kv_pairContext)
            else:
                return self.getTypedRuleContext(jacParser.Kv_pairContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_dict_val

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDict_val" ):
                listener.enterDict_val(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDict_val" ):
                listener.exitDict_val(self)




    def dict_val(self):

        localctx = jacParser.Dict_valContext(self, self._ctx, self.state)
        self.enterRule(localctx, 156, self.RULE_dict_val)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 929
            self.match(jacParser.LBRACE)
            self.state = 938
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.STRING:
                self.state = 930
                self.kv_pair()
                self.state = 935
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==jacParser.COMMA:
                    self.state = 931
                    self.match(jacParser.COMMA)
                    self.state = 932
                    self.kv_pair()
                    self.state = 937
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 940
            self.match(jacParser.RBRACE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Kv_pairContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def COLON(self):
            return self.getToken(jacParser.COLON, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_kv_pair

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterKv_pair" ):
                listener.enterKv_pair(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitKv_pair" ):
                listener.exitKv_pair(self)




    def kv_pair(self):

        localctx = jacParser.Kv_pairContext(self, self._ctx, self.state)
        self.enterRule(localctx, 158, self.RULE_kv_pair)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 942
            self.match(jacParser.STRING)
            self.state = 943
            self.match(jacParser.COLON)
            self.state = 944
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SpawnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def KW_SPAWN(self):
            return self.getToken(jacParser.KW_SPAWN, 0)

        def spawn_object(self):
            return self.getTypedRuleContext(jacParser.Spawn_objectContext,0)


        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_spawn

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpawn" ):
                listener.enterSpawn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpawn" ):
                listener.exitSpawn(self)




    def spawn(self):

        localctx = jacParser.SpawnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 160, self.RULE_spawn)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 946
            self.match(jacParser.KW_SPAWN)
            self.state = 948
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,105,self._ctx)
            if la_ == 1:
                self.state = 947
                self.expression()


            self.state = 950
            self.spawn_object()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Spawn_objectContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def node_spawn(self):
            return self.getTypedRuleContext(jacParser.Node_spawnContext,0)


        def walker_spawn(self):
            return self.getTypedRuleContext(jacParser.Walker_spawnContext,0)


        def graph_spawn(self):
            return self.getTypedRuleContext(jacParser.Graph_spawnContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_spawn_object

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpawn_object" ):
                listener.enterSpawn_object(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpawn_object" ):
                listener.exitSpawn_object(self)




    def spawn_object(self):

        localctx = jacParser.Spawn_objectContext(self, self._ctx, self.state)
        self.enterRule(localctx, 162, self.RULE_spawn_object)
        try:
            self.state = 955
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,106,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 952
                self.node_spawn()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 953
                self.walker_spawn()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 954
                self.graph_spawn()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Node_spawnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def node_ref(self):
            return self.getTypedRuleContext(jacParser.Node_refContext,0)


        def edge_ref(self):
            return self.getTypedRuleContext(jacParser.Edge_refContext,0)


        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_node_spawn

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNode_spawn" ):
                listener.enterNode_spawn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNode_spawn" ):
                listener.exitNode_spawn(self)




    def node_spawn(self):

        localctx = jacParser.Node_spawnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 164, self.RULE_node_spawn)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 958
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.T__2) | (1 << jacParser.T__4) | (1 << jacParser.T__5) | (1 << jacParser.T__6))) != 0) or _la==jacParser.MINUS:
                self.state = 957
                self.edge_ref()


            self.state = 960
            self.node_ref()
            self.state = 962
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,108,self._ctx)
            if la_ == 1:
                self.state = 961
                self.spawn_ctx()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Graph_spawnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def edge_ref(self):
            return self.getTypedRuleContext(jacParser.Edge_refContext,0)


        def graph_ref(self):
            return self.getTypedRuleContext(jacParser.Graph_refContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_graph_spawn

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGraph_spawn" ):
                listener.enterGraph_spawn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGraph_spawn" ):
                listener.exitGraph_spawn(self)




    def graph_spawn(self):

        localctx = jacParser.Graph_spawnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 166, self.RULE_graph_spawn)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 964
            self.edge_ref()
            self.state = 965
            self.graph_ref()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Walker_spawnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def walker_ref(self):
            return self.getTypedRuleContext(jacParser.Walker_refContext,0)


        def spawn_ctx(self):
            return self.getTypedRuleContext(jacParser.Spawn_ctxContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_walker_spawn

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWalker_spawn" ):
                listener.enterWalker_spawn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWalker_spawn" ):
                listener.exitWalker_spawn(self)




    def walker_spawn(self):

        localctx = jacParser.Walker_spawnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 168, self.RULE_walker_spawn)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 967
            self.walker_ref()
            self.state = 969
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,109,self._ctx)
            if la_ == 1:
                self.state = 968
                self.spawn_ctx()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Spawn_ctxContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def spawn_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Spawn_assignContext)
            else:
                return self.getTypedRuleContext(jacParser.Spawn_assignContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_spawn_ctx

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpawn_ctx" ):
                listener.enterSpawn_ctx(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpawn_ctx" ):
                listener.exitSpawn_ctx(self)




    def spawn_ctx(self):

        localctx = jacParser.Spawn_ctxContext(self, self._ctx, self.state)
        self.enterRule(localctx, 170, self.RULE_spawn_ctx)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 971
            self.match(jacParser.LPAREN)
            self.state = 980
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.NAME:
                self.state = 972
                self.spawn_assign()
                self.state = 977
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==jacParser.COMMA:
                    self.state = 973
                    self.match(jacParser.COMMA)
                    self.state = 974
                    self.spawn_assign()
                    self.state = 979
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 982
            self.match(jacParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Filter_ctxContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(jacParser.LPAREN, 0)

        def RPAREN(self):
            return self.getToken(jacParser.RPAREN, 0)

        def filter_compare(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Filter_compareContext)
            else:
                return self.getTypedRuleContext(jacParser.Filter_compareContext,i)


        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_filter_ctx

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFilter_ctx" ):
                listener.enterFilter_ctx(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFilter_ctx" ):
                listener.exitFilter_ctx(self)




    def filter_ctx(self):

        localctx = jacParser.Filter_ctxContext(self, self._ctx, self.state)
        self.enterRule(localctx, 172, self.RULE_filter_ctx)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 984
            self.match(jacParser.LPAREN)
            self.state = 993
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.NAME:
                self.state = 985
                self.filter_compare()
                self.state = 990
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==jacParser.COMMA:
                    self.state = 986
                    self.match(jacParser.COMMA)
                    self.state = 987
                    self.filter_compare()
                    self.state = 992
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 995
            self.match(jacParser.RPAREN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Spawn_assignContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def EQ(self):
            return self.getToken(jacParser.EQ, 0)

        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_spawn_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSpawn_assign" ):
                listener.enterSpawn_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSpawn_assign" ):
                listener.exitSpawn_assign(self)




    def spawn_assign(self):

        localctx = jacParser.Spawn_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 174, self.RULE_spawn_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 997
            self.match(jacParser.NAME)
            self.state = 998
            self.match(jacParser.EQ)
            self.state = 999
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Filter_compareContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def cmp_op(self):
            return self.getTypedRuleContext(jacParser.Cmp_opContext,0)


        def expression(self):
            return self.getTypedRuleContext(jacParser.ExpressionContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_filter_compare

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFilter_compare" ):
                listener.enterFilter_compare(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFilter_compare" ):
                listener.exitFilter_compare(self)




    def filter_compare(self):

        localctx = jacParser.Filter_compareContext(self, self._ctx, self.state)
        self.enterRule(localctx, 176, self.RULE_filter_compare)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1001
            self.match(jacParser.NAME)
            self.state = 1002
            self.cmp_op()
            self.state = 1003
            self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Any_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TYP_STRING(self):
            return self.getToken(jacParser.TYP_STRING, 0)

        def TYP_INT(self):
            return self.getToken(jacParser.TYP_INT, 0)

        def TYP_FLOAT(self):
            return self.getToken(jacParser.TYP_FLOAT, 0)

        def TYP_LIST(self):
            return self.getToken(jacParser.TYP_LIST, 0)

        def TYP_DICT(self):
            return self.getToken(jacParser.TYP_DICT, 0)

        def TYP_BOOL(self):
            return self.getToken(jacParser.TYP_BOOL, 0)

        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def KW_EDGE(self):
            return self.getToken(jacParser.KW_EDGE, 0)

        def KW_TYPE(self):
            return self.getToken(jacParser.KW_TYPE, 0)

        def getRuleIndex(self):
            return jacParser.RULE_any_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAny_type" ):
                listener.enterAny_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAny_type" ):
                listener.exitAny_type(self)




    def any_type(self):

        localctx = jacParser.Any_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 178, self.RULE_any_type)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1005
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.TYP_STRING) | (1 << jacParser.TYP_INT) | (1 << jacParser.TYP_FLOAT) | (1 << jacParser.TYP_LIST) | (1 << jacParser.TYP_DICT) | (1 << jacParser.TYP_BOOL) | (1 << jacParser.KW_TYPE) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_graphContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def dot_stmt_list(self):
            return self.getTypedRuleContext(jacParser.Dot_stmt_listContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def KW_DIGRAPH(self):
            return self.getToken(jacParser.KW_DIGRAPH, 0)

        def KW_STRICT(self):
            return self.getToken(jacParser.KW_STRICT, 0)

        def dot_id(self):
            return self.getTypedRuleContext(jacParser.Dot_idContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_graph

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_graph" ):
                listener.enterDot_graph(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_graph" ):
                listener.exitDot_graph(self)




    def dot_graph(self):

        localctx = jacParser.Dot_graphContext(self, self._ctx, self.state)
        self.enterRule(localctx, 180, self.RULE_dot_graph)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1008
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.KW_STRICT:
                self.state = 1007
                self.match(jacParser.KW_STRICT)


            self.state = 1010
            _la = self._input.LA(1)
            if not(_la==jacParser.KW_GRAPH or _la==jacParser.KW_DIGRAPH):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 1012
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0):
                self.state = 1011
                self.dot_id()


            self.state = 1014
            self.match(jacParser.LBRACE)
            self.state = 1015
            self.dot_stmt_list()
            self.state = 1016
            self.match(jacParser.RBRACE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_stmt_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_stmt(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_stmtContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_stmtContext,i)


        def SEMI(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.SEMI)
            else:
                return self.getToken(jacParser.SEMI, i)

        def getRuleIndex(self):
            return jacParser.RULE_dot_stmt_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_stmt_list" ):
                listener.enterDot_stmt_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_stmt_list" ):
                listener.exitDot_stmt_list(self)




    def dot_stmt_list(self):

        localctx = jacParser.Dot_stmt_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 182, self.RULE_dot_stmt_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1024
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_SUBGRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.LBRACE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0):
                self.state = 1018
                self.dot_stmt()
                self.state = 1020
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.SEMI:
                    self.state = 1019
                    self.match(jacParser.SEMI)


                self.state = 1026
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_node_stmt(self):
            return self.getTypedRuleContext(jacParser.Dot_node_stmtContext,0)


        def dot_edge_stmt(self):
            return self.getTypedRuleContext(jacParser.Dot_edge_stmtContext,0)


        def dot_attr_stmt(self):
            return self.getTypedRuleContext(jacParser.Dot_attr_stmtContext,0)


        def dot_id(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_idContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_idContext,i)


        def EQ(self):
            return self.getToken(jacParser.EQ, 0)

        def dot_subgraph(self):
            return self.getTypedRuleContext(jacParser.Dot_subgraphContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_stmt" ):
                listener.enterDot_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_stmt" ):
                listener.exitDot_stmt(self)




    def dot_stmt(self):

        localctx = jacParser.Dot_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 184, self.RULE_dot_stmt)
        try:
            self.state = 1035
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,118,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 1027
                self.dot_node_stmt()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 1028
                self.dot_edge_stmt()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 1029
                self.dot_attr_stmt()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 1030
                self.dot_id()
                self.state = 1031
                self.match(jacParser.EQ)
                self.state = 1032
                self.dot_id()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 1034
                self.dot_subgraph()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_attr_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_attr_list(self):
            return self.getTypedRuleContext(jacParser.Dot_attr_listContext,0)


        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def KW_EDGE(self):
            return self.getToken(jacParser.KW_EDGE, 0)

        def getRuleIndex(self):
            return jacParser.RULE_dot_attr_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_attr_stmt" ):
                listener.enterDot_attr_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_attr_stmt" ):
                listener.exitDot_attr_stmt(self)




    def dot_attr_stmt(self):

        localctx = jacParser.Dot_attr_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 186, self.RULE_dot_attr_stmt)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1037
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 1038
            self.dot_attr_list()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_attr_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LSQUARE(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.LSQUARE)
            else:
                return self.getToken(jacParser.LSQUARE, i)

        def RSQUARE(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.RSQUARE)
            else:
                return self.getToken(jacParser.RSQUARE, i)

        def dot_a_list(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_a_listContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_a_listContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_dot_attr_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_attr_list" ):
                listener.enterDot_attr_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_attr_list" ):
                listener.exitDot_attr_list(self)




    def dot_attr_list(self):

        localctx = jacParser.Dot_attr_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 188, self.RULE_dot_attr_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1045 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 1040
                self.match(jacParser.LSQUARE)
                self.state = 1042
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0):
                    self.state = 1041
                    self.dot_a_list()


                self.state = 1044
                self.match(jacParser.RSQUARE)
                self.state = 1047 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==jacParser.LSQUARE):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_a_listContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_id(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_idContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_idContext,i)


        def EQ(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.EQ)
            else:
                return self.getToken(jacParser.EQ, i)

        def COMMA(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COMMA)
            else:
                return self.getToken(jacParser.COMMA, i)

        def getRuleIndex(self):
            return jacParser.RULE_dot_a_list

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_a_list" ):
                listener.enterDot_a_list(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_a_list" ):
                listener.exitDot_a_list(self)




    def dot_a_list(self):

        localctx = jacParser.Dot_a_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 190, self.RULE_dot_a_list)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1057 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 1049
                self.dot_id()
                self.state = 1052
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.EQ:
                    self.state = 1050
                    self.match(jacParser.EQ)
                    self.state = 1051
                    self.dot_id()


                self.state = 1055
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==jacParser.COMMA:
                    self.state = 1054
                    self.match(jacParser.COMMA)


                self.state = 1059 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not ((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0)):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_edge_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_edgeRHS(self):
            return self.getTypedRuleContext(jacParser.Dot_edgeRHSContext,0)


        def dot_node_id(self):
            return self.getTypedRuleContext(jacParser.Dot_node_idContext,0)


        def dot_subgraph(self):
            return self.getTypedRuleContext(jacParser.Dot_subgraphContext,0)


        def dot_attr_list(self):
            return self.getTypedRuleContext(jacParser.Dot_attr_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_edge_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_edge_stmt" ):
                listener.enterDot_edge_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_edge_stmt" ):
                listener.exitDot_edge_stmt(self)




    def dot_edge_stmt(self):

        localctx = jacParser.Dot_edge_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 192, self.RULE_dot_edge_stmt)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1063
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [jacParser.KW_GRAPH, jacParser.KW_NODE, jacParser.KW_EDGE, jacParser.FLOAT, jacParser.STRING, jacParser.INT, jacParser.NAME]:
                self.state = 1061
                self.dot_node_id()
                pass
            elif token in [jacParser.KW_SUBGRAPH, jacParser.LBRACE]:
                self.state = 1062
                self.dot_subgraph()
                pass
            else:
                raise NoViableAltException(self)

            self.state = 1065
            self.dot_edgeRHS()
            self.state = 1067
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.LSQUARE:
                self.state = 1066
                self.dot_attr_list()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_edgeRHSContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_edgeop(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_edgeopContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_edgeopContext,i)


        def dot_node_id(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_node_idContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_node_idContext,i)


        def dot_subgraph(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_subgraphContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_subgraphContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_dot_edgeRHS

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_edgeRHS" ):
                listener.enterDot_edgeRHS(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_edgeRHS" ):
                listener.exitDot_edgeRHS(self)




    def dot_edgeRHS(self):

        localctx = jacParser.Dot_edgeRHSContext(self, self._ctx, self.state)
        self.enterRule(localctx, 194, self.RULE_dot_edgeRHS)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1074 
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while True:
                self.state = 1069
                self.dot_edgeop()
                self.state = 1072
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [jacParser.KW_GRAPH, jacParser.KW_NODE, jacParser.KW_EDGE, jacParser.FLOAT, jacParser.STRING, jacParser.INT, jacParser.NAME]:
                    self.state = 1070
                    self.dot_node_id()
                    pass
                elif token in [jacParser.KW_SUBGRAPH, jacParser.LBRACE]:
                    self.state = 1071
                    self.dot_subgraph()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 1076 
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if not (_la==jacParser.T__3 or _la==jacParser.T__7):
                    break

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_edgeopContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return jacParser.RULE_dot_edgeop

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_edgeop" ):
                listener.enterDot_edgeop(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_edgeop" ):
                listener.exitDot_edgeop(self)




    def dot_edgeop(self):

        localctx = jacParser.Dot_edgeopContext(self, self._ctx, self.state)
        self.enterRule(localctx, 196, self.RULE_dot_edgeop)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1078
            _la = self._input.LA(1)
            if not(_la==jacParser.T__3 or _la==jacParser.T__7):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_node_stmtContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_node_id(self):
            return self.getTypedRuleContext(jacParser.Dot_node_idContext,0)


        def dot_attr_list(self):
            return self.getTypedRuleContext(jacParser.Dot_attr_listContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_node_stmt

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_node_stmt" ):
                listener.enterDot_node_stmt(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_node_stmt" ):
                listener.exitDot_node_stmt(self)




    def dot_node_stmt(self):

        localctx = jacParser.Dot_node_stmtContext(self, self._ctx, self.state)
        self.enterRule(localctx, 198, self.RULE_dot_node_stmt)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1080
            self.dot_node_id()
            self.state = 1082
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.LSQUARE:
                self.state = 1081
                self.dot_attr_list()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_node_idContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dot_id(self):
            return self.getTypedRuleContext(jacParser.Dot_idContext,0)


        def dot_port(self):
            return self.getTypedRuleContext(jacParser.Dot_portContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_node_id

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_node_id" ):
                listener.enterDot_node_id(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_node_id" ):
                listener.exitDot_node_id(self)




    def dot_node_id(self):

        localctx = jacParser.Dot_node_idContext(self, self._ctx, self.state)
        self.enterRule(localctx, 200, self.RULE_dot_node_id)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1084
            self.dot_id()
            self.state = 1086
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.COLON:
                self.state = 1085
                self.dot_port()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_portContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COLON(self, i:int=None):
            if i is None:
                return self.getTokens(jacParser.COLON)
            else:
                return self.getToken(jacParser.COLON, i)

        def dot_id(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(jacParser.Dot_idContext)
            else:
                return self.getTypedRuleContext(jacParser.Dot_idContext,i)


        def getRuleIndex(self):
            return jacParser.RULE_dot_port

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_port" ):
                listener.enterDot_port(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_port" ):
                listener.exitDot_port(self)




    def dot_port(self):

        localctx = jacParser.Dot_portContext(self, self._ctx, self.state)
        self.enterRule(localctx, 202, self.RULE_dot_port)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1088
            self.match(jacParser.COLON)
            self.state = 1089
            self.dot_id()
            self.state = 1092
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.COLON:
                self.state = 1090
                self.match(jacParser.COLON)
                self.state = 1091
                self.dot_id()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_subgraphContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LBRACE(self):
            return self.getToken(jacParser.LBRACE, 0)

        def dot_stmt_list(self):
            return self.getTypedRuleContext(jacParser.Dot_stmt_listContext,0)


        def RBRACE(self):
            return self.getToken(jacParser.RBRACE, 0)

        def KW_SUBGRAPH(self):
            return self.getToken(jacParser.KW_SUBGRAPH, 0)

        def dot_id(self):
            return self.getTypedRuleContext(jacParser.Dot_idContext,0)


        def getRuleIndex(self):
            return jacParser.RULE_dot_subgraph

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_subgraph" ):
                listener.enterDot_subgraph(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_subgraph" ):
                listener.exitDot_subgraph(self)




    def dot_subgraph(self):

        localctx = jacParser.Dot_subgraphContext(self, self._ctx, self.state)
        self.enterRule(localctx, 204, self.RULE_dot_subgraph)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1098
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==jacParser.KW_SUBGRAPH:
                self.state = 1094
                self.match(jacParser.KW_SUBGRAPH)
                self.state = 1096
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0):
                    self.state = 1095
                    self.dot_id()




            self.state = 1100
            self.match(jacParser.LBRACE)
            self.state = 1101
            self.dot_stmt_list()
            self.state = 1102
            self.match(jacParser.RBRACE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dot_idContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def NAME(self):
            return self.getToken(jacParser.NAME, 0)

        def STRING(self):
            return self.getToken(jacParser.STRING, 0)

        def INT(self):
            return self.getToken(jacParser.INT, 0)

        def FLOAT(self):
            return self.getToken(jacParser.FLOAT, 0)

        def KW_GRAPH(self):
            return self.getToken(jacParser.KW_GRAPH, 0)

        def KW_NODE(self):
            return self.getToken(jacParser.KW_NODE, 0)

        def KW_EDGE(self):
            return self.getToken(jacParser.KW_EDGE, 0)

        def getRuleIndex(self):
            return jacParser.RULE_dot_id

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDot_id" ):
                listener.enterDot_id(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDot_id" ):
                listener.exitDot_id(self)




    def dot_id(self):

        localctx = jacParser.Dot_idContext(self, self._ctx, self.state)
        self.enterRule(localctx, 206, self.RULE_dot_id)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1104
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << jacParser.KW_GRAPH) | (1 << jacParser.KW_NODE) | (1 << jacParser.KW_EDGE))) != 0) or ((((_la - 93)) & ~0x3f) == 0 and ((1 << (_la - 93)) & ((1 << (jacParser.FLOAT - 93)) | (1 << (jacParser.STRING - 93)) | (1 << (jacParser.INT - 93)) | (1 << (jacParser.NAME - 93)))) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



    def sempred(self, localctx:RuleContext, ruleIndex:int, predIndex:int):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[58] = self.atom_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def atom_sempred(self, localctx:AtomContext, predIndex:int):
            if predIndex == 0:
                return self.precpred(self._ctx, 5)
         




